indexing
	description: "Default implementation of MML_BAG[G]"

class
	MML_DEFAULT_BAG[G]

inherit
	MML_BAG[G]
		undefine
			functional_negated, functional_ored, functional_anded, functional_xored,
			functional_implied, functional_curried_implementation,
			functional_curried_adapter, functional_curried_result_adapter,
			functional_iterated, functional_iterated_adapter, functional_untyped, copy
		end

	MML_DEFAULT_FUNCTION[G, INTEGER]
		rename
			inversed as functional_inversed,
			relational_inversed as inversed,
			evaluated as multiplicity,
			is_defined as is_member,
			image as occurrences,
			has_mapping as has_multiplicity,
			intersected as set_intersected,
			united as set_united,
			difference as set_difference,
			subtracted as set_subtracted,
			relational_extended as set_extended,
			pruned as set_pruned,
			cardinality as set_cardinality,
			do_if as functional_do_if,
			do_all as functional_do_all,
			applied as functional_applied,
			for_all as functional_for_all,
			there_exists as functional_there_exists,
			negated as functional_negated,
			ored as functional_ored,
			xored as functional_xored,
			anded as functional_anded,
			implied as functional_implied,
			curried_adapter as functional_curried_adapter,
			curried_result_adapter as functional_curried_result_adapter,
			curried_implementation as functional_curried_implementation,
			iterated_adapter as functional_iterated_adapter,
			iterated as functional_iterated,
			untyped as functional_untyped,
			random_sequence as set_random_sequence
		export
			{MML_ANY}
				set_intersected, set_united, set_subtracted, set_extended, set_pruned, set_difference,
				is_subset, is_proper_subset, is_superset, is_proper_superset, functional_untyped,
				relational_intersected, relational_united, functional_inversed, set_random_sequence
		undefine
			equals
		redefine
			is_compatible, make_from_other
		end

create
	make_empty, make_from_other

feature {NONE} -- Initialization

	make_from_other (other : MML_BAG[G]) is
			-- Create a new object containing the elements from `other'.
		do
			Precursor (other)
		end

feature {MML_ANY} -- Type

	is_compatible (other : MML_ANY) : BOOLEAN is
			-- Is `other' mathematically compatible to `current' ?
		do
			Result := other.conforms_to (create {MML_DEFAULT_BAG[G]}.make_empty)
		ensure then
			implementation_of_compatible : other.conforms_to (create {MML_DEFAULT_BAG[G]}.make_empty) implies Result
		end

feature -- Comparison

	equals (other: MML_BAG[G]): BOOLEAN is
			-- Is `other' mathematically equivalent to `current' ?
		do
			Result := Precursor (other)
		end

feature -- Conversion

	random_sequence : MML_SEQUENCE[G] is
			-- The bag `current' as random sequence.
		local
			accumulator : MML_DEFAULT_SEQUENCE[G]
		do
			create accumulator.make_empty
			do_all (agent accumulator.append_element (?))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

feature -- Measurement

	cardinality : INTEGER is
			-- The cardinality of `current'
		do
			Result := accumulated_integer_value (0, agent infix_plus (?, ?), agent multiplicity (?))
		ensure then
			definition_of_cardinality : Result.is_equal (accumulated_integer_value (0, agent infix_plus (?, ?), agent multiplicity (?)))
		end

feature -- Basic Operations

	intersected (other: MML_BAG[G]) : like Current is
			-- The intersection of `current' and `other'.
		do
			create Result.make_empty
			other.domain.intersected (domain).do_all (agent accumulate_intersected_element (?, Result, other, Current))
		ensure then
			result_not_void : Result /= Void
		end

	united (other: MML_BAG[G]) : like Current is
			-- The union of `current' and `other'.
		do
			create Result.make_empty
			other.domain.united (domain).do_all (agent accumulate_united_element (?, Result, other, Current))
		ensure then
			result_not_void : Result /= Void
		end

	subtracted, removed (other: MML_BAG[G]) : like Current is
			-- The difference of `current' and `other'.
		do
			Result := domain_anti_restricted (other.domain)
			other.domain.intersected (domain).do_all (agent accumulate_subtracted_element (?, Result, other, Current))
		ensure then
			result_not_void : Result /= Void
		end

	added (other: like Current) : like Current is
			-- The bag `current' with all elements from `other'.
		do
			Result := clone (Current)
			other.domain.united (domain).do_all (agent accumulate_added_element (?, Result, other, Current))
		ensure then
			result_not_void : Result /= Void
		end

	extended (other: G) : like Current is
			-- The bag `current' extended with `other'.
		do
			if is_member (other) then
				Result := domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other)).set_extended (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, multiplicity (other) + 1))
			else
				Result := set_extended (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, 1))
			end
		ensure then
			implementation_of_extended_not_member : not is_member (other) implies Result.is_equal (set_extended (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, 1)))
			implementation_of_extended_is_member : is_member (other) implies Result.is_equal (domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other)).set_extended (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, multiplicity (other) + 1)))
			result_not_void : Result /= Void
		end

	pruned (other: G) : like Current is
			-- The bag `current' with `other' removed.
		do
			if is_member (other) then
				Result := domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other))
				if multiplicity (other) > 1 then
					Result := Result.set_extended (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, multiplicity (other) - 1))	
				end
			else
				Result := clone (Current)
			end
		ensure then
			implementation_of_pruned_not_member : not is_member (other) implies Result.is_equal (Current)
			implementation_of_pruned_is_multiple_member : is_member (other) and multiplicity (other) > 1 implies Result.is_equal (domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other)).set_extended (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, multiplicity (other) - 1))	)
			implementation_of_pruned_is_member : is_member (other) and multiplicity (other) = 1 implies Result.is_equal (domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

feature -- Quantifiers

	there_exists (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : BOOLEAN is
			-- Does `current' contain an element which satisfies `predicate'?
		do
			Result := domain.for_all (predicate)
		ensure then
			implementation_of_there_exists : Result.is_equal (domain.for_all (predicate))
		end

	for_all (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : BOOLEAN is
			-- Does `current' contain only elements which satisfy `predicate'?
		do
			Result := domain.for_all (predicate)
		ensure then
			implementation_of_for_all : Result.is_equal (domain.for_all (predicate))
		end

feature -- Processing

	applied (action: PROCEDURE [ANY, TUPLE [G]]) : like Current is
			-- The object with `action' applied to every element of `current'.
		do
			Result := clone (Current)
			Result.do_all (action)
		ensure then
			result_not_void : Result /= Void
		end

feature {MML_ANY} -- Implementation

	do_all (action: PROCEDURE [ANY, TUPLE [G]]) is
			-- Apply `action' to every element of `current'.
			-- CAUTION: This feature is not pure
		do
			domain.do_all (action)
		end

	do_if (action: PROCEDURE [ANY, TUPLE [G]]; predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) is
			-- Apply `action' to every element that satisfies `predicate'.
			-- CAUTION: This feature is not pure
		do
			domain.do_if (action, predicate)
		end

feature {NONE} -- Implementation

	accumulate_added_element (other : G; accumulator : MML_DEFAULT_BAG[G]; one : MML_BAG[G]; two : MML_BAG[G]) is
			-- Accumulate `other' with the added multiplicity of `one' and `two' in `accumulator'.
		require
			accumulator_not_void : accumulator /= Void
			one_not_void : one /= Void
			two_not_void : two /= Void
			other_not_void : other /= Void
			other_member_in_bags : one.is_member (other) or two.is_member (other)
			not_accumulator_member : not accumulator.is_member (other)
		do
			if one.is_member (other) and two.is_member (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, one.multiplicity (other) + (two.multiplicity (other))))
			elseif one.is_member (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, one.multiplicity (other)))
			elseif two.is_member (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, two.multiplicity (other)))
			end
		end

	accumulate_united_element (other : G; accumulator : MML_DEFAULT_BAG[G]; one : MML_BAG[G]; two : MML_BAG[G]) is
			-- Accumulate `other' with the maximal multiplicity of `one' and `two' in `accumulator'.
		require
			accumulator_not_void : accumulator /= Void
			one_not_void : one /= Void
			two_not_void : two /= Void
			other_not_void : other /= Void
			other_member_in_bags : one.is_member (other) or two.is_member (other)
			not_accumulator_member : not accumulator.is_member (other)
		do
			if one.is_member (other) and two.is_member (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, one.multiplicity (other).max (two.multiplicity (other))))
			elseif one.is_member (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, one.multiplicity (other)))
			elseif two.is_member (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, two.multiplicity (other)))
			end
		end

	accumulate_intersected_element (other : G; accumulator : MML_DEFAULT_BAG[G]; one : MML_BAG[G]; two : MML_BAG[G]) is
			-- Accumulate `other' with the minimal multiplicity of `one' and `two' in `accumulator'.
		require
			accumulator_not_void : accumulator /= Void
			one_not_void : one /= Void
			two_not_void : two /= Void
			other_not_void : other /= Void
			other_member_in_one : one.is_member (other)
			other_member_in_two : two.is_member (other)
			not_accumulator_member : not accumulator.is_member (other)
		do
			accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, one.multiplicity (other).min (two.multiplicity (other))))
		end

	accumulate_subtracted_element (other : G; accumulator : MML_DEFAULT_BAG[G]; one : MML_BAG[G]; two : MML_BAG[G]) is
			-- Accumulate `other' with the multiplicity difference of `one' and `two' in `accumulator'.
		require
			accumulator_not_void : accumulator /= Void
			one_not_void : one /= Void
			two_not_void : two /= Void
			other_not_void : other /= Void
			other_member_in_one : one.is_member (other)
			other_member_in_two : two.is_member (other)
			not_accumulator_member : not accumulator.is_member (other)
		do
			if one.multiplicity (other) > two.multiplicity (other) then
				accumulator.extend (create {MML_DEFAULT_PAIR[G, INTEGER]}.make_from (other, one.multiplicity (other) - two.multiplicity (other)))
			end
		end

	accumulate_integer_value (operator : FUNCTION [ANY, TUPLE [INTEGER, INTEGER], INTEGER]; mapping: FUNCTION [ANY, TUPLE [G], INTEGER]; element : G) is
			-- Accumulate the integer value of `element' using `mapping' and `operator'.
		require
			operator_not_void : operator /= Void
			mapping_not_void : mapping /= Void
			element_not_void : element /= Void
			accumulator_not_void : accumulator_integer /= Void
		do
			accumulator_integer := operator.item ([accumulator_integer, mapping.item ([element])])
		end

	accumulated_integer_value (start : INTEGER; operator : FUNCTION [ANY, TUPLE [INTEGER, INTEGER], INTEGER]; mapping: FUNCTION [ANY, TUPLE [G], INTEGER]) : INTEGER is
			-- The accumulated integer value over `current' starting from `start' using `mapping' and `operator'.
		local
			backup : INTEGER
		do
			backup := accumulator_integer
			
			accumulator_integer := start
			domain.do_all (agent accumulate_integer_value (operator, mapping, ?))

			Result := accumulator_integer
			accumulator_integer := backup
		end

	accumulator_integer : INTEGER
		-- The integer accumulator of `current'.

	infix_plus (left : INTEGER; right : INTEGER) : INTEGER is
			-- Encapsulation of the infix "+" operator.
		obsolete
			"This wrapper should be removed as soon as agents on operators are implemented"
		do
			Result := left + right
		ensure
			definition_of_plus : Result.is_equal (left + right)
		end

end -- class MML_DEFAULT_BAG