indexing
	description: "Default implementation of MML_FUNCTION[G, H]"

class
	MML_DEFAULT_FUNCTION [G, H]

inherit
	MML_FUNCTION[G, H]
		undefine
			copy
		end

	MML_DEFAULT_RELATION[G, H]
		rename
			intersected as relational_intersected,
			united as relational_united,
			is_member as has_mapping,
			inversed as relational_inversed,
			extended as relational_extended,
			curried as curried_implementation
		export
			{MML_DEFAULT_FUNCTION}
				relational_extended
		undefine
			untyped
		redefine
			is_compatible, composed, make_from_other
		end

create
	make_empty, make_from_other, make_from_relation

feature {NONE} -- Initialization

	make_from_other (other : MML_FUNCTION[G, H]) is
			-- Creates a new object containing the elements from `other'.
		do
			Precursor (other)
		end

	make_from_relation (other : MML_RELATION[G, H]) is
			-- Create a new function representing `other'.
		do
			make_empty
			other.do_all (agent extend (?))
		end

feature {MML_ANY} -- Type

	is_compatible (other : MML_ANY) : BOOLEAN is
			-- Is `other' mathematically compatible to `current' ?
		do
			Result := other.conforms_to (create {MML_DEFAULT_FUNCTION[G, H]}.make_empty)
		ensure then
			implementation_of_compatible : other.conforms_to (create {MML_DEFAULT_FUNCTION[G, H]}.make_empty) implies Result
		end

feature -- Status Report

	is_injective : BOOLEAN is
			-- Is `current' injective?
		do
			Result := for_all (agent is_accumulator_member (?, create {MML_DEFAULT_SET[H]}.make_empty))
		ensure then
			implementation_of_injective : Result.is_equal (for_all (agent is_accumulator_member (?, create {MML_DEFAULT_SET[H]}.make_empty)))
		end

	is_defined (argument : G) : BOOLEAN is
			-- Is `current' defined for `argument'?
		do
			Result := domain.is_member (argument)
		ensure then
			implementation_of_defined : domain.is_member (argument) implies Result
		end

feature -- Basic Operations

	intersected (other: MML_FUNCTION[G, H]) : MML_FUNCTION[G, H] is
			-- The intersection of `current' and `other'.
		do
			Result := relational_intersected (other)
		ensure then
			implementation_of_intersected : Result.is_equal (relational_intersected (other))
			result_not_void : Result /= Void
		end

	united (other: MML_FUNCTION[G, H]) : MML_FUNCTION[G, H] is
			-- The overriding union of `current' and `other'.
		do
			create {like Current} Result.make_from_relation (domain_anti_restricted (other.domain).relational_united (other))
		ensure then
			result_not_void : Result /= Void
		end

feature -- Inversion

	inversed : MML_FUNCTION[H, G] is
			-- The inverse function of `current'.
		do
			create {MML_DEFAULT_FUNCTION[H, G]} Result.make_from_relation (relational_inversed)
		ensure then
			result_not_void : Result /= Void
		end

feature -- Composition

	composed (other : MML_FUNCTION[H, ANY]) : MML_FUNCTION[G, ANY] is
			-- The function composed of `current' and `other'.
		local
			accumulator : MML_DEFAULT_FUNCTION[G, ANY]
		do
			create accumulator.make_empty
			range_restricted (range.intersected (other.domain)).do_all (agent accumulate_composed_pairs (?, accumulator, other.domain_restricted (range.intersected (other.domain))))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

feature -- Closures

	sequential_closure (other : ANY) : MML_SEQUENCE[ANY] is
			-- The sequenctial closure of `current' starting from `other'.
		local
			accumulator : MML_DEFAULT_SEQUENCE[ANY]
			iterator : ANY
		do
			from
				create accumulator.make_empty
				iterator := other
			until
				not untyped.domain.is_member (iterator)
			loop
				accumulator.append_element (iterator)
				iterator := untyped.evaluated (iterator)
			end
			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

feature {NONE} -- Implementation

	is_accumulator_member (element : MML_PAIR[G, H]; accumulator : MML_DEFAULT_SET[H]) : BOOLEAN is
			-- Is `element' already in `accumulator'?
		require
			accumulator_not_void : accumulator /= Void
			element_not_void : element /= Void
		do
			if accumulator.is_member (element.second) then
				Result := True
			else
				accumulator.extend (element.second)
			end
		end

end -- class MML_DEFAULT_FUNCTION