indexing
	description: "Default implementation of MML_GRAPH[G]"

class
	MML_DEFAULT_GRAPH[G]

inherit
	MML_GRAPH[G]
		undefine
			is_compatible
		end

	MML_DEFAULT_PAIR[MML_SET[G], MML_RELATION[G, G]]
		rename
			first as nodes,
			second as edges,
			element as component
		export
			{MML_ANY}
				is_identity
		undefine
			untyped, is_graph_valued, is_compatible
		redefine
			make_from_other
		end

create
	make_empty, make_from, make_from_other

feature {NONE} -- Initialization

	make_empty is
			-- Create a new empty graph.
		do
			make_from (create {MML_DEFAULT_SET[G]}.make_empty, create {MML_DEFAULT_RELATION[G, G]}.make_empty)
		end

	make_from_other (other : MML_GRAPH[G]) is
			-- Create a new object containing the elements from `other'.
		do
			make_from (other.nodes, other.edges)
		end

feature {MML_ANY} -- Type

	is_compatible (other : MML_ANY) : BOOLEAN is
			-- Is `other' mathematically compatible to `current' ?
		do
			Result := other.conforms_to (create {MML_DEFAULT_GRAPH[G]}.make_empty)
		ensure then
			implementation_of_compatible : other.conforms_to (create {MML_DEFAULT_GRAPH[G]}.make_empty) implies Result
		end

feature -- Basic Operations

	edge_extended (other : MML_PAIR[G, G]) : like Current is
			-- The graph `current' extended with `other'.
		do
			create Result.make_from (nodes, edges.extended (other))
		ensure then
			implementation_of_edge_extended : Result.is_equal (create {MML_DEFAULT_GRAPH[G]}.make_from (nodes, edges.extended (other)))
			result_not_void : Result /= Void
		end

	node_extended (other : G) : like Current is
			-- The graph `current' extended with `other'.
		do
			create Result.make_from (nodes.extended (other), edges)
		ensure then
			implementation_of_node_extended : Result.is_equal (create {MML_DEFAULT_GRAPH[G]}.make_from (nodes.extended (other), edges))
			result_not_void : Result /= Void
		end

	edge_pruned (other : MML_PAIR[G, G]) : like Current is
			-- The graph `current' with `other' removed.
		do
			create Result.make_from (nodes, edges.pruned (other))
		ensure then
			implementation_of_edge_pruned : Result.is_equal (create {MML_DEFAULT_GRAPH[G]}.make_from (nodes, edges.pruned (other)))
			result_not_void : Result /= Void
		end

	node_pruned (other : G) : like Current is
			-- The graph `current' with `other' removed.
		do
			create Result.make_from (nodes.pruned (other), edges.domain_element_anti_restricted (other).intersected (edges.range_element_anti_restricted (other)))
		ensure then
			implementation_of_node_pruned : Result.is_equal (create {MML_DEFAULT_GRAPH[G]}.make_from (nodes.pruned (other), edges.domain_element_anti_restricted (other).intersected (edges.range_element_anti_restricted (other))))
			result_not_void : Result /= Void
		end

end -- class MML_DEFAULT_GRAPH