indexing
	description: "Default implementation of MML_PAIR[G, H]"

class
	MML_DEFAULT_PAIR [G, H]

inherit
	MML_PAIR[G, H]

create
	make_from, make_from_other

feature {NONE} -- Initialization

	make_from (one : G; two : H) is
			-- Create a new pair with `one' and `two'.
		do
			internal_first := one
			internal_second := two
		end

	make_from_other (other : MML_PAIR[G, H]) is
			-- Create a new object containing the elements from `other'.
		do
			make_from (other.first, other.second)
		end

feature -- Conversion

	identity_pair : MML_PAIR[like Current, like Current] is
			-- The identity pair of `current'.
		do
			Result := create {MML_DEFAULT_PAIR[like Current, like Current]}.make_from (Current, Current)
		ensure then
			implementation_of_identity_pair : Result.is_equal (create {MML_DEFAULT_PAIR[like Current, like Current]}.make_from (Current, Current))
			result_not_void : Result /= Void
		end

feature -- Access

	first : G is
			-- The first element of `current'.
		do
			Result := internal_first
		ensure then
			implementation_of_first : Result.is_equal (internal_first)
			result_not_void : Result /= Void
		end

	second : H is
			-- The second element of `current'.
		do
			Result := internal_second
		ensure then
			implementation_of_second : Result.is_equal (internal_second)
			result_not_void : Result /= Void
		end

feature -- Inversion

	inversed : MML_PAIR[H, G] is
			-- The inverse pair of `current'.
		do
			Result := create {MML_DEFAULT_PAIR[H, G]}.make_from (second, first)
		ensure then
			implementation_of_reversed : Result.is_equal (create {MML_DEFAULT_PAIR[H, G]}.make_from (second, first))
			result_not_void : Result /= Void
		end

feature {NONE} -- Implementation

	internal_first : G
		-- The implementation of `first'
	
	internal_second : H
		-- The implementation of `second'

invariant
	internal_first_not_void : internal_first /= Void
	internal_second_not_void : internal_second /= Void

	definition_of_first : first.is_equal (internal_first)
	definition_of_second : second.is_equal (internal_second)

end -- class MML_DEFAULT_PAIR