indexing
	description: "Default implementation of MML_POWERSET[G]"

class
	MML_DEFAULT_POWERSET[G]

inherit
	MML_POWERSET[G]
		undefine
			copy
		end

	MML_DEFAULT_SET[MML_SET[G]]
		rename
			for_all as for_all_sets,
			there_exists as there_exists_set,
			negated as set_negated,
			anded as set_anded,
			ored as set_ored,
			xored as set_xored,
			implied as set_implied,
			iterated as set_iterated,
			iterated_adapter as set_iterated_adapter,
			curried_adapter as set_curried_adapter,
			curried_result_adapter as set_curried_result_adapter,
			curried as set_curried
		undefine
			is_compatible, set_negated,
			set_ored, set_anded, set_implied,
			set_xored, set_curried, set_curried_adapter,
			set_curried_result_adapter, set_iterated,
			set_iterated_adapter
		redefine
			make_from_other
		end

create
	make_empty, make_from_element, make_from_set, make_from_other

feature {NONE} -- Initialization

	make_from_other (other : MML_POWERSET[G]) is
			-- Create a new object containing the elements from `other'.
		do
			Precursor (other)
		end

feature {MML_ANY} -- Type

	is_compatible (other : MML_ANY) : BOOLEAN is
			-- Is `other' mathematically compatible to `current' ?
		do
			Result := other.conforms_to (create {MML_DEFAULT_SET[MML_SET[G]]}.make_empty)
		ensure then
			implementation_of_compatible : other.conforms_to (create {MML_DEFAULT_SET[MML_SET[G]]}.make_empty) implies Result
		end

feature -- Generalized Operations

	generalized_united : MML_SET[G] is
			-- The generalized union of `current'.
		local
			accumulator : MML_DEFAULT_SET[G]
		do
			create accumulator.make_empty
			do_all (agent {MML_DEFAULT_SET[G]}.do_all (agent accumulator.extend (?)))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

	generalized_intersected : MML_SET[G] is
			-- The generalized intersection of `current'.
		local
			dummy: MML_SET[G]
		do
			if is_empty then
				create {MML_DEFAULT_SET[G]} Result.make_empty
			elseif cardinality = 1 then
				Result := clone (any_element)
			else
				dummy := any_element
				Result := dummy.intersected (pruned (dummy).generalized_intersected)
			end
		ensure then
			result_not_void : Result /= Void
		end

feature -- Quantifiers

	there_exists (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : BOOLEAN is
			-- Does `current' contain an element which satisfies `predicate'?
		do
			Result := generalized_united.there_exists (predicate)
		ensure then
			implementation_of_there_exists: Result.is_equal (generalized_united.there_exists (predicate))
		end

	for_all (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : BOOLEAN is
			-- Does `current' contain only elements which satisfy `predicate'?
		do
			Result := generalized_united.for_all (predicate)
		ensure then
			implementation_of_for_all: Result.is_equal (generalized_united.for_all (predicate))
		end

end -- class MML_DEFAULT_POWERSET