indexing
	description: "Default implementation of MML_RELATION[G, H]"

class
	MML_DEFAULT_RELATION[G, H]

inherit
	MML_RELATION[G, H]
		undefine
			copy
		end

	MML_DEFAULT_SET[MML_PAIR[G, H]]
		rename
			composed as composed_implementation
		undefine
			equals, united, untyped, is_compatible, is_relation_valued, is_function_valued
		end

create
	make_empty, make_from_element, make_from_set, make_from_other, make_from_sequences

feature {NONE} -- Initialization

	make_from_sequences (dom: MML_SEQUENCE[G]; ran: MML_SEQUENCE[H]) is
			-- Create a new relation from `dom' and `ran'.
		local
			index : INTEGER
		do
			make_empty
			from
				index := dom.lower_bound
			until
				index > dom.upper_bound
			loop
				extend (create {MML_DEFAULT_PAIR[G, H]}.make_from (dom.element (index), ran.element (index)))
				index := index + 1
			end
		end

feature {MML_ANY} -- Type

	is_compatible (other : MML_ANY) : BOOLEAN is
			-- Is `other' mathematically compatible to `current' ?
		do
			Result := other.conforms_to (create {MML_DEFAULT_SET[MML_PAIR[G, H]]}.make_empty)
		ensure then
			implementation_of_compatible : other.conforms_to (create {MML_DEFAULT_SET[MML_PAIR[G, H]]}.make_empty) implies Result
		end

feature -- Basic Operations

	united (other: MML_SET[MML_PAIR[G, H]]) : MML_RELATION[G, H] is
			-- The union of `current' and `other'.
		do
			Result := clone (Current)
			other.do_all (agent Result.extend (?))
		ensure then
			result_not_void : Result /= Void
		end

feature -- Projections

	image_element (other : G) : MML_SET[H] is
			-- The projected range set of `current' through `other'.
		do
			Result := image (create {MML_DEFAULT_SET[G]}.make_from_element (other))
		ensure then
			implementation_of_image_element : Result.is_equal (image (create {MML_DEFAULT_SET[G]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	anti_image_element (other : H) : MML_SET[G] is
			-- The projected domain set of `current' through `other'.
		do
			Result := anti_image (create {MML_DEFAULT_SET[H]}.make_from_element (other))
		ensure then
			implementation_of_image_element : Result.is_equal (anti_image (create {MML_DEFAULT_SET[H]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	domain_element_anti_restricted (other : G) : like Current is
			-- A relation with pairs of `current' having its first element not equivalent to `other'.
		do
			Result := domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other))
		ensure then
			implementation_of_domain_element_anti_restricted : Result.is_equal (domain_anti_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	domain_element_restricted (other : G) : like Current is
			-- A relation with pairs of `current' having its first element equivalent to `other'.
		do
			Result := domain_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other))
		ensure then
			implementation_of_domain_element_restricted : Result.is_equal (domain_restricted (create {MML_DEFAULT_SET[G]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	domain_restricted (restriction : MML_SET[G]) : like Current is
			-- A relation with pairs of `current' having its first element in `restriction'.
		local
			accumulator : like Current
		do
			create accumulator.make_empty
			do_if (agent accumulator.extend (?), agent is_domain_member (?, restriction))

			Result := accumulator
		ensure then
			implementation_of_domain_restricted : not Result.there_exists (agent is_domain_member (?, restriction))
			result_not_void : Result /= Void
		end

	range_element_anti_restricted (other : H) : like Current is
			-- A relation with pairs of `current' having its second element not equivalent `other'.
		do
			Result := range_anti_restricted (create {MML_DEFAULT_SET[H]}.make_from_element (other))
		ensure then
			implementation_of_range_element_anti_restricted : Result.is_equal (range_anti_restricted (create {MML_DEFAULT_SET[H]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	range_element_restricted (other : H) : like Current is
			-- A relation with pairs of `current' having its second element equivalent to `other'.
		do
			Result := range_restricted (create {MML_DEFAULT_SET[H]}.make_from_element (other))
		ensure then
			implementation_of_range_element_restricted : Result.is_equal (range_restricted (create {MML_DEFAULT_SET[H]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	range_restricted (restriction : MML_SET[H]) : like Current is
			-- A relation with pairs of `current' having its second element in `restriction'.
		local
			accumulator : like Current
		do
			create accumulator.make_empty
			do_if (agent accumulator.extend (?), agent is_range_member (?, restriction))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

feature -- Inversion

	inversed : MML_RELATION[H, G] is
			-- The reverse relation of `current'.
		do
			create {MML_DEFAULT_RELATION[H, G]} Result.make_from_set (mapped_to_inverse (agent {MML_PAIR[G, H]}.inversed))
		ensure then
			implementation_of_inversed : Result.is_equal (create {MML_DEFAULT_RELATION[H, G]}.make_from_set (mapped_to_inverse (agent {MML_PAIR[G, H]}.inversed)))
			result_not_void : Result /= Void
		end

feature -- Composition

	composed (other : MML_RELATION[H, ANY]) : MML_RELATION[G, ANY] is
			-- The relation composed of `current' and `other'.
		local
			accumulator : MML_DEFAULT_RELATION[G, ANY]
		do
			create accumulator.make_empty
			range_restricted (range.intersected (other.domain)).do_all (agent accumulate_composed_pairs (?, accumulator, other.domain_restricted (range.intersected (other.domain))))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

feature -- Closures

	transitive_closure : MML_RELATION[ANY, ANY] is
			-- The transitive closure of `current'.
		do
			Result := transitive_closure_implementation (False)
		ensure then
			result_not_void : Result /= Void
		end

	reflexive_transitive_closure : MML_RELATION[ANY, ANY] is
			-- The reflexive-transitive closure of `current'.
		do
			Result := transitive_closure_implementation (True)
		ensure then
			result_not_void : Result /= Void
		end

feature -- Mapping

	mapped_to_inverse (mapping: FUNCTION [ANY, TUPLE [MML_PAIR[G, H]], MML_PAIR[H, G]]) : MML_SET[MML_PAIR[H, G]] is
			-- The object containing all inverse elements in `current' mapped using the `mapping' function.
		local
			accumulator : MML_DEFAULT_SET[MML_PAIR[H, G]]
		do
			create accumulator.make_empty
			do_all (agent accumulate_inverse_element (?, mapping, accumulator))

			Result := accumulator
		ensure then
			same_cardinality : Result.cardinality.is_equal (cardinality)
			result_not_void : Result /= Void
		end

	mapped_to_first (mapping: FUNCTION [ANY, TUPLE [MML_PAIR[G, H]], G]) : MML_SET[G] is
			-- The object containing all first elements in `current' mapped using the `mapping' function.
		local
			accumulator : MML_DEFAULT_SET[G]
		do
			create accumulator.make_empty
			do_all (agent accumulate_first_element (?, mapping, accumulator))

			Result := accumulator
		ensure then
			same_cardinality : Result.cardinality.is_equal (cardinality)
			result_not_void : Result /= Void
		end

	mapped_to_second (mapping: FUNCTION [ANY, TUPLE [MML_PAIR[G, H]], H]) : MML_SET[H] is
			-- The object containing all second elements in `current' mapped using the `mapping' function.
		local
			accumulator : MML_DEFAULT_SET[H]
		do
			create accumulator.make_empty
			do_all (agent accumulate_second_element (?, mapping, accumulator))

			Result := accumulator
		ensure then
			same_cardinality : Result.cardinality.is_equal (cardinality)
			result_not_void : Result /= Void
		end

feature {NONE} -- Implementation

	is_domain_member (pair : MML_PAIR[G, H]; other : MML_SET[G]) : BOOLEAN is
			-- Is the first element of `pair' a member of `other'?
		require
			other_not_void : other /= Void
			pair_not_void : pair /= Void
		do
			Result := other.is_member (pair.first)
		end

	is_range_member (pair : MML_PAIR[G, H]; other : MML_SET[H]) : BOOLEAN is
			-- Is the second element of `pair' a member of `other'?
		require
			other_not_void : other /= Void
			pair_not_void : pair /= Void
		do
			Result := other.is_member (pair.second)
		end

	accumulate_composed_pairs (element : MML_PAIR[G, H]; accumulator : MML_DEFAULT_SET[MML_PAIR[G, ANY]]; restriction : MML_RELATION[H, ANY]) is
		-- Accumulate the pairs composed from `restriction' with `element' in `accumulator'.
		require
			accumulator_not_void : accumulator /= Void
			restriction_not_void : restriction /= Void
			element_not_void : element /= Void
		do
			restriction.image (create {MML_DEFAULT_SET[H]}.make_from_element (element.second)).do_all (agent accumulate_composed_pair (?, element.first, accumulator))
		end

	accumulate_composed_pair (target : ANY; source : G; accumulator : MML_DEFAULT_SET[MML_PAIR[G, ANY]]) is
			-- Accumulate the pair based on `source' and `target' in `accumulator'.
		require
			accumulator_not_void : accumulator /= Void
			source_not_void : source /= Void
			target_not_void : target /= Void
		do
			accumulator.extend (create {MML_DEFAULT_PAIR[like source, like target]}.make_from (source, target))
		end

	accumulate_first_element (element : MML_PAIR[G, H]; transformator : FUNCTION [ANY, TUPLE [MML_PAIR[G, H]], G]; accumulator : MML_EXTENDIBLE[G]) is
			-- Accumulate the first element of `element' computed by `transformator' in `accumulator'.
		require
			element_not_void : element /= Void
			transformator_not_void : transformator /= Void
			accumulator_not_void : accumulator /= Void
		do
			accumulator.extend (transformator.item ([element]))
		end

	accumulate_second_element (element : MML_PAIR[G, H]; transformator : FUNCTION [ANY, TUPLE [MML_PAIR[G, H]], H]; accumulator : MML_EXTENDIBLE[H]) is
			-- Accumulate the second element of `element' computed by `transformator' in `accumulator'.
		require
			element_not_void : element /= Void
			transformator_not_void : transformator /= Void
			accumulator_not_void : accumulator /= Void
		do
			accumulator.extend (transformator.item ([element]))
		end

	accumulate_inverse_element (element : MML_PAIR[G, H]; transformator : FUNCTION [ANY, TUPLE [MML_PAIR[G, H]], MML_PAIR[H, G]]; accumulator : MML_EXTENDIBLE[MML_PAIR[H, G]]) is
			-- Accumulate the inverse of `element' computed by `transformator' in `accumulator'.
		require
			element_not_void : element /= Void
			transformator_not_void : transformator /= Void
			accumulator_not_void : accumulator /= Void
		do
			accumulator.extend (transformator.item ([element]))
		end

	transitive_closure_implementation (reflexive : BOOLEAN) : MML_RELATION[ANY, ANY] is
			-- Implementation of Warshall's transitive closure algorithm.
		local
			matrix : ARRAY2[BOOLEAN]
			order : MML_SEQUENCE[ANY]
			i, j, k : INTEGER
			this, accumulator : MML_DEFAULT_RELATION[ANY, ANY]
		do
			this := Current
			order := domain.random_sequence

			create matrix.make (order.count, order.count)
			if reflexive then
				from
					i := order.lower_bound
				until
					i > order.lower_bound
				loop
					matrix.put (True, i, i)
					i := i + 1
				end
			end

			from
				i := order.lower_bound
			until
				i > order.upper_bound
			loop
				from
					j := order.lower_bound
				until
					j > order.upper_bound
				loop
					if this.is_member (create {MML_DEFAULT_PAIR[ANY, ANY]}.make_from (order.element (i), order.element (j))) then
						matrix.put (True, i, j)
					end
					j := j + 1
				end
				i := i + 1
			end
			
			from
				i := order.lower_bound
			until
				i > order.upper_bound
			loop
				from
					j := order.lower_bound
				until
					j > order.upper_bound
				loop
					from
						k := order.lower_bound
					until
						k > order.upper_bound
					loop
						matrix.put (matrix.item (i, j) or (matrix.item (i, k) and matrix.item (k, j)), i, j)
						k := k + 1
					end
					j := j + 1
				end
				i := i + 1
			end

			create accumulator.make_empty
			from
				i := order.lower_bound
			until
				i > order.upper_bound
			loop
				from
					j := order.lower_bound
				until
					j > order.upper_bound
				loop
					if matrix.item (i, j) then
						accumulator.extend (create {MML_DEFAULT_PAIR[ANY, ANY]}.make_from (order.element (i), order.element (j)))
					end
					j := j + 1
				end
				i := i + 1
			end
			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

end -- class MML_DEFAULT_RELATION