indexing
	description: "Default implementation of MML_SEQUENCE[G]"

class
	MML_DEFAULT_SEQUENCE[G]

inherit
	MML_SEQUENCE[G]
		undefine
			functional_anded, functional_ored, functional_xored,
			functional_negated, functional_implied,
			composed_adapter, composed_implementation,
			functional_curried_implementation, functional_curried_adapter,
			functional_curried_result_adapter, functional_iterated,
			functional_iterated_adapter, functional_untyped, copy
		end

	MML_DEFAULT_FUNCTION[INTEGER, G]
		rename
			evaluated as element,
			cardinality as count,
			make_from_element as make_from_pair,
			make_from_relation as make_from_functional_relation,
			has_mapping as at_index,
			image as elements,
			anti_image_element as indexes_element,
			anti_image as indexes_elements,
			pruned as relational_pruned,
			subtracted as relational_subtracted,
			do_if as functional_do_if,
			do_all as functional_do_all,
			applied as functional_applied,
			for_all as functional_for_all,
			there_exists as functional_there_exists,
			negated as functional_negated,
			ored as functional_ored,
			xored as functional_xored,
			anded as functional_anded,
			implied as functional_implied,
			filtered as functional_filtered,
			is_partition as is_set_partition,
			range_restricted as functional_range_restricted,
			range_anti_restricted as functional_range_anti_restricted,
			range_element_anti_restricted as functional_range_element_anti_restricted,
			domain_restricted as functional_domain_restricted,
			domain_anti_restricted as functional_domain_anti_restricted,
			domain_element_anti_restricted as functional_domain_element_anti_restricted,
			curried_adapter as functional_curried_adapter,
			curried_result_adapter as functional_curried_result_adapter,
			curried_implementation as functional_curried_implementation,
			iterated_adapter as functional_iterated_adapter,
			iterated as functional_iterated,
			untyped as functional_untyped
		export
			{NONE}
				functional_filtered, functional_range_restricted, functional_range_anti_restricted,
				functional_range_element_anti_restricted, functional_domain_restricted,
				functional_domain_anti_restricted, functional_domain_element_anti_restricted,
				intersected, united, relational_intersected, relational_united,
				image_element, make_from_pair
			{MML_ANY}
				is_subset, is_proper_subset, is_superset, is_proper_superset, functional_untyped,
				relational_pruned, relational_subtracted, relational_inversed
		undefine
			equals, is_defined, is_sequence_valued
		redefine
			is_compatible, make_empty, make_from_other
		end

create
	make_empty, make_empty_lower, make_from_element, make_from_element_lower, make_from_relation, make_from_relation_lower, make_from_other, make_from_other_lower

feature {NONE} -- Initialization

	make_empty is
			-- Create a new empty sequence.
		do
			Precursor
			internal_lower_bound := 1
		end

	make_empty_lower (lower : INTEGER) is
			-- Create a new empty sequence starting at `lower'.
		do
			internal_lower_bound := lower
		end

	make_from_element (other : G) is
			-- Create a new sequence containing `other'.
		do
			make_empty
			extend (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (lower_bound, other))
		ensure then
			implementation_of_make_from_element : is_equal ((create {like Current}.make_empty).appended (other))
		end

	make_from_element_lower (other : G; lower : INTEGER) is
			-- Create a new sequence containing `other' at `lower'.
		do
			make_empty_lower (lower)
			extend (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (lower_bound, other))
		ensure then
			implementation_of_make_from_element_lower : is_equal ((create {like Current}.make_empty_lower (lower)).appended (other))
		end

	make_from_relation (other : MML_RELATION[INTEGER, G]) is
			-- Create a new sequence containing `other'.
		do
			make_empty;
			(create {MML_DEFAULT_RELATION[INTEGER, G]}.make_from_other (other)).do_all (agent extend (?))
		ensure then
			implementation_of_make_from_relation : other.for_all (agent at_index (?))
		end

	make_from_relation_lower (other : MML_RELATION[INTEGER, G]; lower : INTEGER) is
			-- Create a new sequence containing `other' starting at `lower'.
		do
			make_empty_lower (lower)
			(create {like Current}.make_from_relation (other)).do_all (agent append_element (?))
		end

	make_from_other (other : MML_SEQUENCE[G]) is
			-- Create a new object containing the elements from `other'.
		do
			Precursor (other)
		end

	make_from_other_lower (other : MML_SEQUENCE[G]; lower : INTEGER) is
			-- Create a new object containing the elements from `other' starting at `lower'.
		do
			make_empty_lower (lower)
			other.do_all (agent append_element (?))
		end

feature {MML_ANY} -- Type

	is_compatible (other : MML_ANY) : BOOLEAN is
			-- Is `other' mathematically compatible to `current' ?
		do
			Result := other.conforms_to (create {MML_DEFAULT_SEQUENCE[G]}.make_empty)
		ensure then
			implementation_of_compatible : other.conforms_to (create {MML_DEFAULT_SEQUENCE[G]}.make_empty) implies Result
		end

feature -- Comparison

	equals (other: MML_SEQUENCE[G]): BOOLEAN is
			-- Is `other' mathematically equivalent to `current' ?
		do
			Result := Precursor (other)
		end

feature -- Properties

	lower_bound : INTEGER is
			-- The lower index bound of `current'.
		do
			Result := internal_lower_bound
		end

feature -- Basic Operations

	subtracted (other: MML_SET[G]) : like Current is
			-- The difference of `current' and `other'.
		do
			create Result.make_empty
			range.do_if (agent Result.append_element (?), agent negated (?, agent other.is_member (?)))
		ensure then
			result_not_void : Result /= Void
		end

	extended_at (other: G; position : INTEGER) : like Current is
			-- The sequence `current' extended with `other' at `position'.
		do
			if position.is_equal (lower_bound) then
				Result := prepended (other)
			elseif position.is_equal (upper_bound) then
				Result := appended (other)
			else
				Result := interval (lower_bound, position - 1).appended (other).concatenated (interval (position, upper_bound))
			end
		ensure then
			result_not_void : Result /= Void
		end

	pruned (other: G) : like Current is
			-- The sequence `current' with all occurrences of `other' removed.
		do
			Result := subtracted (create {MML_DEFAULT_SET[G]}.make_from_element (other))
		ensure then
			implementation_of_pruned : Result.is_equal (subtracted (create {MML_DEFAULT_SET[G]}.make_from_element (other)))
			result_not_void : Result /= Void
		end

	pruned_at (position : INTEGER) : like Current is
			-- The sequence `current' with the element at `position' pruned.
		do
			if position.is_equal (lower_bound) then
				Result := tail
			elseif position.is_equal (upper_bound) then
				Result := front
			else
				Result := interval (lower_bound, position - 1).concatenated (interval (position + 1, upper_bound))
			end
		ensure then
			result_not_void : Result /= Void
		end

	replaced (other: G; position: INTEGER) : like Current is
			-- The sequence `current' with the element at `position' replaced by `other'.
		do
			create Result.make_from_relation ((create {MML_DEFAULT_RELATION[INTEGER, G]}.make_from_other (Current)).domain_element_anti_restricted (position).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (position, other)))
		ensure then
			implementation_of_replaced : Result.is_equal ((create {like Current}.make_from_relation ((create {MML_DEFAULT_RELATION[INTEGER, G]}.make_from_other (Current)).domain_element_anti_restricted (position).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (position, other)))))
			result_not_void : Result /= Void
		end

feature -- Projections

	domain_restricted (restriction : MML_SET[INTEGER]) : like Current is
			-- A relation with pairs of `current' having its first element in `restriction'.
		local
			accumulator : like Current
		do
			create accumulator.make_empty
			do_if (agent accumulator.append_element (?), agent (elements (restriction)).is_member (?))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

	range_restricted (restriction : MML_SET[G]) : like Current is
			-- A relation with pairs of `current' having its second element in `restriction'.
		local
			accumulator : like Current
		do
			create accumulator.make_empty
			do_if (agent accumulator.append_element (?), agent restriction.is_member (?))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

	indexes_element_ordered (other : G) : MML_SEQUENCE[INTEGER] is
			-- The projected domain set of `current' through `other'.
		do
			Result := indexes_elements_ordered (create {MML_DEFAULT_SET[G]}.make_from_element (other))
		ensure then
			implementation_of_indexes_element_ordered : Result.is_equal (indexes_elements_ordered (create {MML_DEFAULT_SET[G]}.make_from_element (other)))
		end

	indexes_elements_ordered (projection : MML_SET[G]) : MML_SEQUENCE[INTEGER] is
			-- The projected domain set of `current' through `projection'.
		local
			accumulator : MML_DEFAULT_SEQUENCE[INTEGER]
		do
			create accumulator.make_empty
			functional_do_all (agent accumulate_indexes (?, accumulator, projection))

			Result := accumulator
		ensure then
			result_not_void : Result /= Void
		end

feature -- Reversal

	reversed : like Current is
			-- The sequence with all elements of `current', but in reversed order.
		local
			stack : STACK[G]
		do
			create {ARRAYED_STACK[G]} stack.make (count)
			create Result.make_empty

			do_all (agent stack.put (?))

			stack.linear_representation.do_all (agent Result.append_element (?))
		ensure then
			result_not_void : Result /= Void
		end

feature -- Decomposition

	interval (lower_index : INTEGER; upper_index : INTEGER) : like Current is
			-- The interval from `lower_index' to `upper_index'.
		local
			offset : INTEGER
		do
			create Result.make_empty
			from
				offset := lower_index
			until
				offset > upper_index
			loop
				Result.append_element (element (offset))
				offset := offset + 1
			end
		end

feature -- Concatenation

	appended (other : G) : like Current is
			-- The sequence `current' with `other' appended.
		do
			Result := clone (Current)
			Result.append_element (other)
		ensure then
			implementation_of_appended : Result.at_index (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (upper_bound + 1, other))
			result_not_void : Result /= Void
		end

	prepended (other : G) : like Current is
			-- The sequence `current' prepended with `other'.
		do
			Result := (create {like Current}.make_from_element (other)).concatenated (Current)
		ensure then
			implementation_of_prepended : Result.at_index (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (lower_bound, other))
			result_not_void : Result /= Void
		end

	concatenated (other : MML_SEQUENCE[G]) : like Current is
			-- The concatenation of `current' and `other'.
		do
			Result := clone (Current)
			other.do_all (agent Result.append_element (?))
		ensure then
			result_not_void : Result /= Void
		end

feature -- Partition

	is_partition (other : MML_SEQUENCE[MML_SEQUENCE[G]]) : BOOLEAN is
			-- Is `other' a partition of `current'?
		local
			accumulator : MML_DEFAULT_SEQUENCE[G]
		do
			create accumulator.make_empty
			other.do_all (agent accumulator.append_sequence (?))

			Result := is_equal (accumulator)
		end

feature -- Quantifiers

	there_exists (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : BOOLEAN is
			-- Does `current' contain an element which satisfies `predicate'?
		do
			Result := range.there_exists (predicate)
		ensure then
			implementation_of_there_exists : Result.is_equal (range.there_exists (predicate))
		end

	for_all (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : BOOLEAN is
			-- Does `current' contain only elements which satisfy `predicate'?
		do
			Result := range.for_all (predicate)
		ensure then
			implementation_of_for_all : Result.is_equal (range.for_all (predicate))
		end

feature -- Processing

	applied (action: PROCEDURE [ANY, TUPLE [G]]) : like Current is
			-- The object with `action' applied to every element of `current'.
		do
			Result := clone (Current)
			Result.do_all (action)
		ensure then
			result_not_void : Result /= Void
		end

feature -- Filtering

	filtered (predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) : like Current is
			-- The object containing all elements in `current' not filtered by `predicate'.
		do
			create Result.make_empty
			do_if (agent Result.append_element (?), predicate)
		ensure then
			result_not_void : Result /= Void
		end

feature {MML_ANY} -- Processing

	do_all (action: PROCEDURE [ANY, TUPLE [G]]) is
			-- Apply `action' to every element of `current'.
			-- CAUTION: This feature is not pure
		local
			offset : INTEGER
		do
			from
				offset := lower_bound
			until
				offset > upper_bound
			loop
				action.call ([element (offset)])
			end
		end

	do_if (action: PROCEDURE [ANY, TUPLE [G]]; predicate: FUNCTION [ANY, TUPLE [G], BOOLEAN]) is
			-- Apply `action' to every element that satisfies `predicate'.
			-- CAUTION: This feature is not pure
		local
			offset : INTEGER
		do
			from
				offset := lower_bound
			until
				offset > upper_bound
			loop
				if predicate.item ([element (offset)]) then
					action.call ([element (offset)])	
				end
			end
		end

feature {MML_ANY} -- Implementation

	append_element (other : G) is
			-- Append `other' to `current'.
			-- CAUTION: This feature is not pure
		require
			other_not_void : other /= Void
		do
			extend (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (upper_bound + 1, other))
		end

	append_sequence (other : MML_SEQUENCE[G]) is
			-- Append `other' to `current'.
			-- CAUTION: This feature is not pure
		require
			other_not_void : other /= Void
		do
			other.do_all (agent append_element (?))
		end

feature {NONE} -- Implementation

	internal_lower_bound : INTEGER
		-- The internal lower bound

	is_reversed (entry : MML_PAIR[INTEGER, G]) : BOOLEAN is
			-- Is `entry' a reversed version of an entry in `current'?
		do
			Result := at_index (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (upper_bound - entry.first + 1, entry.second))
		end

	accumulate_indexes (pair : MML_PAIR[INTEGER, G]; accumulator : MML_DEFAULT_SEQUENCE[INTEGER]; projection : MML_SET[G]) is
			-- Accumulate the index of the element in `pair' if it is a member of `projection' in `accumulator'.
		require
			pair_not_void : pair /= Void
			accumulator_not_void : accumulator /= Void
			projection_not_void : projection /= Void
		do
			if projection.is_member (pair.second) then
				accumulator.append_element (pair.first)
			end
		end

end -- class MML_DEFAULT_SEQUENCE