indexing
	description: "Mathematical sets that can be mapped to sets of other types"

class
	MML_MAPPABLE_SET[G, H]

inherit
	MML_DEFAULT_SET[G]

	MML_MAPPABLE[G, H]
		undefine
			copy
		select
			mapped
		end

create
	make_empty, make_from_element, make_from_set, make_from_other

feature -- Mapping

	mapped (mapping: FUNCTION [ANY, TUPLE [G], H]) : MML_SET[H] is
			-- The object containing all elements in `current' mapped using the `mapping' function.
		local
			accumulator : MML_DEFAULT_SET[H]
		do
			create accumulator.make_empty
			do_all (agent accumulate_mapped_element (?, mapping, accumulator))

			Result := accumulator
		ensure then
			same_cardinality : Result.cardinality.is_equal (cardinality)
			result_not_void : Result /= Void
		end

feature {NONE} -- Implementation

	accumulate_mapped_element (element : G; transformator : FUNCTION [ANY, TUPLE [G], H]; accumulator : MML_EXTENDIBLE[H]) is
			-- Accumulate the mapping of `element' computed by `transformator' in `accumulator'.
		require
			element_not_void : element /= Void
			transformator_not_void : transformator /= Void
			accumulator_not_void : accumulator /= Void
		do
			accumulator.extend (transformator.item ([element]))
		end

end -- class MML_MAPPABLE_SET