indexing
	description: "Mathematical sets over which a result can be computed using an accumulator"

class
	MML_REDUCEABLE_SET[G -> NUMERIC, H]

inherit
	MML_DEFAULT_SET[H]
		redefine
			copy
		end

	MML_REDUCEABLE[G, H]
		redefine
			copy
		end

create
	make_empty, make_from_element, make_from_set, make_from_other

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			Precursor {MML_DEFAULT_SET} (other)
			accumulator_value := clone (other.accumulator_value)
		end

feature -- Accumulation

	reduced (start : G; operator : FUNCTION [ANY, TUPLE [G, G], G]; mapping: FUNCTION [ANY, TUPLE [H], G]) : G is
			-- The value accumulated over `current' starting from `start' using `mapping' and `operator'.
		do
			accumulator_value := start
			do_all (agent accumulate_result (operator, mapping, ?))
			Result := accumulator_value
		ensure then
			implementation_of_accumulated : Result.is_equal (accumulator_value)
			result_not_void : Result /= Void
		end

feature {MML_REDUCEABLE_SET} -- Implementation

	accumulator_value : G
		-- The accumulator of `current'.

feature {NONE} -- Implementation

	accumulate_result (operator : FUNCTION [ANY, TUPLE [G, G], G]; mapping: FUNCTION [ANY, TUPLE [H], G]; element : H) is
			-- Accumulate the value of `element' using `mapping' and `operator'.
		require
			operator_not_void : operator /= Void
			mapping_not_void : mapping /= Void
			element_not_void : element /= Void
			accumulator_not_void : accumulator_value /= Void
		do
			accumulator_value := operator.item ([accumulator_value, mapping.item ([element])])
		end

end -- class MML_REDUCEABLE_SET