indexing
	description: "Base class of all mathematical objects"

deferred class
	MML_ANY

inherit
	ANY
		export
			{NONE}
				all
			{MML_ANY}
				conforms_to
			{ANY}
				same_type, is_equal,
				clone, deep_clone
		redefine
			is_equal
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' considered equal to `current'?
		do
			Result := equals (other)
		ensure then
			implementation_of_equal : Result.is_equal (equals (other))
		end

	equals (other: MML_ANY): BOOLEAN is
			-- Is `other' mathematically equivalent to `current'?
		require
			other_not_void : other /= Void
		do
			Result := standard_is_equal (other)
		ensure
			implementation_of_equals : standard_is_equal (other) implies Result
			symmetric_equality_relation : Result implies other.equals (Current)
		end

feature -- Type

	is_pair_valued : BOOLEAN is
			-- Is `current' pair-valued?
		do
			Result := False
		end

	is_set_valued : BOOLEAN is
			-- Is `current' set-valued?
		do
			Result := False
		end

	is_relation_valued : BOOLEAN is
			-- Is `current' relation-valued?
		do
			Result := False
		end

	is_function_valued : BOOLEAN is
			-- Is `current' function-valued?
		do
			Result := False
		end

	is_graph_valued : BOOLEAN is
			-- Is `current' graph-valued?
		do
			Result := False
		end

	is_sequence_valued : BOOLEAN is
			-- Is `current' sequence-valued?
		do
			Result := False
		end

	untyped : ANY is
			-- The object `current' as typeless object.
		do
			Result := Current
		ensure
			definition_of_untyped : Result.is_equal (Current)
		end

invariant
	reflexive_equivalence_relation : equals (Current)
	reflexive_equality_relation : is_equal (Current)

	relation_implies_set : is_relation_valued implies is_set_valued
	function_implies_relation : is_function_valued implies is_relation_valued
	graph_implies_function : is_graph_valued implies (is_function_valued and is_pair_valued)

end -- class MML_ANY