indexing
	description: "Mathematical set objects containing set-valued elements"

deferred class
	MML_POWERSET[G]

inherit
	MML_SET[MML_SET[G]]
		rename
			for_all as for_all_sets,
			there_exists as there_exists_set,
			negated as set_negated,
			anded as set_anded,
			ored as set_ored,
			xored as set_xored,
			implied as set_implied,
			iterated as set_iterated,
			iterated_adapter as set_iterated_adapter,
			curried_adapter as set_curried_adapter,
			curried_result_adapter as set_curried_result_adapter,
			curried as set_curried
		undefine
			is_compatible, is_equal, composed, composed_adapter
		select
			for_all_sets, there_exists_set,
			set_negated, set_anded, set_ored,
			set_xored, set_implied,
			set_curried, set_curried_adapter,
			set_curried_result_adapter,
			set_iterated_adapter, set_iterated
		end

	MML_QUANTIFIABLE[G]
		rename
			negated as power_set_negated,
			anded as power_set_anded,
			ored as power_set_ored,
			xored as power_set_xored,
			implied as power_set_implied,
			iterated_adapter as power_set_iterated_adapter,
			iterated as power_set_iterated,
			curried_adapter as power_set_curried_adapter,
			curried_result_adapter as power_set_curried_result_adapter,
			curried as power_set_curried
		end

feature {NONE} -- Initialization

	make_from_other (other : MML_POWERSET[G]) is
			-- Create a new object containing the elements from `other'.
		deferred
		end

feature -- Generalized Operations

	generalized_united : MML_SET[G] is
			-- The generalized union of `current'.
		deferred
		ensure
			definition_of_generalized_united : for_all_sets (agent {MML_SET[G]}.for_all (agent Result.is_member (?)))
		end

	generalized_intersected : MML_SET[G] is
			-- The generalized intersection of `current'.
		deferred
		end

feature -- Disjointness

	is_generalized_disjoint : BOOLEAN is
			-- Is `current' disjoint?
		do
			Result := generalized_intersected.is_empty
		ensure
			definition_of_disjoint : Result.is_equal (generalized_intersected.is_empty)
		end

feature -- Partition

	is_generalized_partition (other : MML_SET[G]) : BOOLEAN is
			-- Is `current' a partition of `other'?
		require
			other_not_void : other /= Void
		do
			Result := is_generalized_disjoint and then other.is_equal (generalized_united)
		ensure
			definition_of_partition : Result.is_equal (is_generalized_disjoint and other.is_equal (generalized_united))
		end

invariant
	definition_of_powerset : for_all_sets (agent {MML_SET[G]}.is_set_valued) and is_set_valued

	disjointness_implies_empty_intersection : is_generalized_disjoint implies generalized_intersected.is_empty
	disjointness_implies_partition : is_generalized_disjoint implies is_generalized_partition (generalized_united)

	definition_of_generalized_union : generalized_united.is_subset (generalized_intersected)

end -- class MML_POWERSET