indexing
	description: "Mathematical objects containing elements in a specified order"

deferred class
	MML_SEQUENCE[G]

inherit
	MML_FUNCTION[INTEGER, G]
		rename
			cardinality as count,
			evaluated as element,
			make_from_element as make_from_pair,
			make_from_relation as make_from_functional_relation,
			has_mapping as at_index,
			image as elements,
			anti_image_element as indexes_element,
			anti_image as indexes_elements,
			pruned as relational_pruned,
			subtracted as relational_subtracted,
			do_if as functional_do_if,
			do_all as functional_do_all,
			applied as functional_applied,
			for_all as functional_for_all,
			there_exists as functional_there_exists,
			negated as functional_negated,
			anded as functional_anded,
			ored as functional_ored,
			xored as functional_xored,
			implied as functional_implied,
			filtered as functional_filtered,
			is_partition as is_set_partition,
			range_restricted as functional_range_restricted,
			range_anti_restricted as functional_range_anti_restricted,
			range_element_anti_restricted as functional_range_element_anti_restricted,
			domain_restricted as functional_domain_restricted,
			domain_anti_restricted as functional_domain_anti_restricted,
			domain_element_anti_restricted as functional_domain_element_anti_restricted,
			curried_adapter as functional_curried_adapter,
			curried_result_adapter as functional_curried_result_adapter,
			curried_implementation as functional_curried_implementation,
			iterated as functional_iterated,
			iterated_adapter as functional_iterated_adapter,
			untyped as functional_untyped
		export
			{NONE}
				functional_filtered, functional_range_restricted, functional_range_anti_restricted,
				functional_range_element_anti_restricted, functional_domain_restricted,
				functional_domain_anti_restricted, functional_domain_element_anti_restricted,
				intersected, united, relational_intersected, relational_united,
				image_element, make_from_pair
			{MML_ANY}
				is_subset, is_proper_subset, is_superset, is_proper_superset, functional_untyped,
				relational_pruned, relational_subtracted, relational_inversed
		undefine
			equals
		redefine
			is_sequence_valued
		select
			functional_for_all, functional_there_exists,
			functional_do_if, functional_do_all,
			functional_applied, functional_negated,
			functional_anded, functional_ored,
			functional_xored, functional_implied,
			functional_filtered, composed_implementation,
			functional_curried_adapter, functional_curried_implementation,
			functional_curried_result_adapter, functional_iterated,
			functional_iterated_adapter, functional_untyped
		end

	MML_TYPEABLE[MML_SEQUENCE[G]]

	MML_PROCESSABLE[G]

	MML_QUANTIFIABLE[G]
		rename
			curried_adapter as sequence_curried_adapter,
			curried_result_adapter as sequence_curried_result_adapter,
			curried as sequence_curried_implementation,
			iterated_adapter as sequence_iterated_adapter,
			iterated as sequence_iterated,
			composed as functional_composed
		undefine
			composed_adapter
		end

	MML_FILTERABLE[G]

feature {NONE} -- Initialization

	make_empty_lower (lower : INTEGER) is
			-- Create a new empty sequence starting at `lower'.
		deferred
		ensure
			definition_of_make_empty : not there_exists (agent is_member(?))
			valid_cardinality : count.is_equal (0)
		end

	make_from_element (other : G) is
			-- Create a new sequence containing `other'.
		deferred
		ensure
			definition_of_make_from_element : range.is_member (other) and element (lower_bound).is_equal (other)
			valid_count : count.is_equal (1)
		end

	make_from_element_lower (other : G; lower : INTEGER) is
			-- Create a new sequence containing `other' at `lower'.
		deferred
		ensure
			definition_of_make_from_element : range.is_member (other) and element (lower_bound).is_equal (other)
			valid_count : count.is_equal (1)
		end

	make_from_relation (other : MML_RELATION[INTEGER, G]) is
			-- Create a new sequence containing `other'.
		require
			other_not_void : other /= Void
			sequential_relation_required : (create {INTEGER_INTERVAL}.make (lower_bound, upper_bound)).for_all (agent domain.anded (?, agent is_defined (?), agent domain.is_member (?)))
		deferred
		ensure
			definition_of_make_from_relation: other.is_equal (Current)
			same_members: other.range.for_all (agent is_member (?))
			same_cardinality: count.is_equal (other.cardinality)
		end

	make_from_relation_lower (other : MML_RELATION[INTEGER, G]; lower : INTEGER) is
			-- Create a new sequence containing `other' starting at `lower'.
		require
			other_not_void : other /= Void
			sequential_relation_required : (create {INTEGER_INTERVAL}.make (lower_bound, upper_bound)).for_all (agent domain.anded (?, agent is_defined (?), agent domain.is_member (?)))
		deferred
		ensure
			definition_of_make_from_relation: other.is_equal (Current)
			same_members: other.range.for_all (agent is_member (?))
			same_cardinality: count.is_equal (other.cardinality)
		end

	make_from_other (other : MML_SEQUENCE[G]) is
			-- Create a new object containing the elements from `other'.
		deferred
		end

	make_from_other_lower (other : MML_SEQUENCE[G]; lower : INTEGER) is
			-- Create a new object containing the elements from `other' starting at `lower'.
		deferred
		end

feature -- Comparison

	equals (other: MML_SEQUENCE[G]): BOOLEAN is
			-- Is `other' mathematically equivalent to `current'?
		deferred
		end

feature -- Type

	untyped : MML_SEQUENCE[ANY] is
			-- The object `current' as typeless object.
		do
			Result := Current
		end

	is_sequence_valued : BOOLEAN is
			-- Is `current' sequence-valued?
		do
			Result := True
		ensure then
			result_is_true : Result
		end

feature -- Properties

	is_member (other: G) : BOOLEAN is
			-- Is `other' a member of `current'?
		require
			other_not_void : other /= Void
		do
			Result := range.is_member (other)
		ensure
			definition_of_member : Result.is_equal (range.is_member (other))
			empty_implies_not_member: is_empty implies not Result
		end

	is_defined (index : INTEGER) : BOOLEAN is
			-- Is `index' defined in `current'?
		do
			Result := (index >= lower_bound) and (index <= upper_bound)
		ensure then
			definition_of_defined : Result.is_equal (index >= lower_bound and index <= upper_bound)
		end

	is_subsequence (other: MML_SEQUENCE[G]) : BOOLEAN is
			-- Is `other' a subsequence of `current'?
		require
			other_not_void : other /= Void
		do
			Result := other.functional_for_all (agent at_index(?))
		ensure
			definition_of_subsequence : Result.is_equal (other.functional_for_all (agent at_index (?)))
			valid_count : Result.is_equal (count >= other.count)
		end

	is_supersequence (other: MML_SEQUENCE[G]) : BOOLEAN is
			-- Is `other' a supersequence of `current'?
		require
			other_not_void : other /= Void
		do
			Result := other.is_subsequence (Current)
		ensure
			definition_of_supersequence : Result.is_equal (other.is_subsequence (Current))
			valid_count : Result.is_equal (other.count >= count)
		end

	is_proper_supersequence (other: MML_SEQUENCE[G]) : BOOLEAN is
			-- Is `other' a proper supersequence of `current'?
		require
			other_not_void : other /= Void
		do
			Result := is_supersequence (other) and not is_equal (other)
		ensure
			definition_of_proper_supersequence : Result.is_equal (is_supersequence (other) and not is_equal (other))
			all_elements_included : Result implies for_all (agent other.is_member(?))
			there_exists_element : Result implies other.there_exists (agent negated (?, agent is_member (?)))
			valid_count : Result.is_equal (other.count > count)
		end

	is_proper_subsequence (other: MML_SEQUENCE[G]) : BOOLEAN is
			-- Is `other' a proper subsequence of `current'?
		require
			other_not_void : other /= Void
		do
			Result := is_subsequence (other) and not is_equal (other)
		ensure
			definition_of_proper_subsequence : Result.is_equal (is_subsequence (other) and not is_equal (other))
			all_elements_included : Result implies other.for_all (agent is_member(?))
			there_exists_element : Result implies there_exists (agent negated (?, agent other.is_member(?)))
			valid_count : Result.is_equal (other.count < count)
		end

	lower_bound : INTEGER is
			-- The lower index bound of `current'.
		deferred
		end

	upper_bound : INTEGER is
			-- The upper index bound of `current'.
		do
			Result := lower_bound + count - 1
		ensure
			definition_of_upper_bound: Result.is_equal (lower_bound + count - 1)
		end

feature -- Basic Operations

	subtracted (other: MML_SET[G]) : like Current is
			-- The sequence `current' with all elements of `other' removed.
		require
			other_not_void : other /= Void
		deferred
		ensure
			definition_of_subtracted : Result.is_equal (filtered (agent negated (?, agent other.is_member (?))))
			disjoint_subtracted_result : Result.range.intersected (other).is_empty
			valid_count : Result.count <= count
		end

	extended_at (other: G; position : INTEGER) : like Current is
			-- The sequence `current' extended with `other' at `position'.
		require
			other_not_void : other /= Void
			defined_position : is_defined (position)
		deferred
		ensure
			definition_of_extended_at: Result.element (position).is_equal (other)
			first_element_extended : position.is_equal (lower_bound) implies Result.is_equal (prepended (other))
			last_element_extended : position.is_equal (upper_bound) implies Result.is_equal (appended (other))
			middle_element_extended : (not position.is_equal (upper_bound) and not position.is_equal (lower_bound)) implies Result.is_equal (interval (lower_bound, position - 1).appended (other).concatenated (interval (position, upper_bound)))
			valid_count : count.is_equal (old count + 1)
		end

	pruned (other: G) : like Current is
			-- The sequence `current' with all occurrences of `other' removed.
		require
			other_not_void : other /= Void
		deferred
		ensure
			definition_of_pruned : not Result.is_member (other)
			valid_count : count <= old count
		end

	pruned_at (position : INTEGER) : like Current is
			-- The sequence `current' with the element at `position' pruned.
		require
			defined_position : is_defined (position)
		deferred
		ensure
			first_element_pruned : position.is_equal (lower_bound) implies Result.is_equal (tail)
			last_element_pruned : position.is_equal (upper_bound) implies Result.is_equal (front)
			middle_element_pruned : (not position.is_equal (upper_bound) and not position.is_equal (lower_bound)) implies Result.is_equal (interval (lower_bound, position - 1).concatenated (interval (position + 1, upper_bound)))
			valid_count : count.is_equal (old count - 1)
		end

	replaced (other: G; position: INTEGER) : like Current is
			-- The sequence `current' with the element at `position' replaced by `other'.
		require
			other_not_void : other /= Void
			defined_position : is_defined (position)
		deferred
		ensure
			definition_of_replaced : element (position).is_equal (other)
			first_element_replaced : position.is_equal (lower_bound) implies Result.is_equal (tail.prepended (other))
			last_element_replaced : position.is_equal (upper_bound) implies Result.is_equal (front.appended (other))
			middle_element_replaced : (not position.is_equal (upper_bound) and not position.is_equal (lower_bound)) implies Result.is_equal (interval (lower_bound, position - 1).appended (other).concatenated (interval (position + 1, upper_bound)))
			valid_count : count.is_equal (old count)
		end

feature -- Projections

	occurrences (other : G) : INTEGER is
			-- The occurrences of `other' in `current'.
		require
			other_not_void: other /= Void
		do
			Result := indexes_element (other).cardinality
		ensure
			definition_of_occurrences : Result.is_equal (indexes_element (other).cardinality)
		end

	domain_restricted (restriction : MML_SET[INTEGER]) : like Current is
			-- A relation with pairs of `current' having its first element in `restriction'.
		require
			restriction_not_void : restriction /= Void
		deferred
		ensure
			definition_of_domain_restricted : Result.for_all (agent (elements (restriction)).is_member (?))
		end

	range_restricted (restriction : MML_SET[G]) : like Current is
			-- A relation with pairs of `current' having its second element in `restriction'.
		require
			restriction_not_void : restriction /= Void
		deferred
		ensure
			definition_of_range_restricted : Result.for_all (agent restriction.is_member (?))
		end

	indexes_element_ordered (other : G) : MML_SEQUENCE[INTEGER] is
			-- The projected domain set of `current' through `other'.
		require
			other_not_void : other /= Void
		deferred
		end

	indexes_elements_ordered (other : MML_SET[G]) : MML_SEQUENCE[INTEGER] is
			-- The projected domain set of `current' through `other'.
		require
			other_not_void : other /= Void
		deferred
		ensure
			definition_of_indexes_elements_ordered : Result.range.is_equal (range_restricted (other).domain)
		end

feature -- Reversal

	reversed : like Current is
			-- The sequence with all elements of `current', but in reversed order.
		deferred
		ensure
			definition_of_reversed : Result.functional_for_all (agent is_reversed (?)) and Result.range.is_equal (range)
		end

feature -- Decomposition

	first : G is
			-- The first element of `current'.
		require
			non_empty : not is_empty
		do
			Result := element (lower_bound)
		ensure
			definition_of_first : Result.is_equal (element (lower_bound))
			result_is_member : is_member (Result)
		end

	last : G is
			-- The last element of `current'.
		require
			non_empty : not is_empty
		do
			Result := element (upper_bound)
		ensure
			definition_of_last : Result.is_equal (element (upper_bound))
			result_is_member : is_member (Result)
		end

	tail : like Current is
			-- The elements of `current' except for the first one.
		do
			Result := domain_restricted (domain.pruned (lower_bound))
		ensure
			definition_of_tail : Result.is_equal (domain_restricted (domain.pruned (lower_bound)))
			result_are_members : Result.for_all (agent is_member (?))
		end

	front : like Current is
			-- The elements of `current' except for the last one.
		do
			Result := domain_restricted (domain.pruned (upper_bound))
		ensure
			definition_of_front : Result.is_equal (domain_restricted (domain.pruned (upper_bound)))
			result_are_members : Result.for_all (agent is_member (?))
		end

	interval (lower : INTEGER; upper : INTEGER) : like Current is
			-- The interval from `lower' to `upper'.
		require
			valid_bounds : is_defined (lower) and is_defined (upper) and upper >= lower
		deferred
		ensure
			definition_of_interval : Result.count.is_equal (upper - lower + 1)
			result_are_members : Result.for_all (agent is_member (?))
		end

feature -- Concatenation

	appended (other : G) : like Current is
			-- The sequence `current' with `other' appended.
		require
			other_not_void : other /= Void
		deferred
		ensure
			definition_of_appended : Result.front.is_equal (Current) and Result.last.is_equal (other)
			valid_count : Result.count.is_equal (count + 1)
		end

	prepended (other : G) : like Current is
			-- The sequence `current' prepended with `other'.
		require
			other_not_void : other /= Void
		deferred
		ensure
			definition_of_prepended : Result.tail.is_equal (Current) and Result.first.is_equal (other)
			valid_count : Result.count.is_equal (count + 1)
		end

	concatenated (other : MML_SEQUENCE[G]) : like Current is
			-- The concatenation of `current' and `other'.
		require
			other_not_void : other /= Void
		deferred
		ensure
			definition_of_concatenated : Result.range.is_equal (range.united (other.range))
			valid_count : Result.count.is_equal (count + other.count)
		end

feature -- Partition

	is_partition (other : MML_SEQUENCE[MML_SEQUENCE[G]]) : BOOLEAN is
			-- Is `other' a partition of `current'?
		require
			other_not_void : other /= Void
		deferred
		end

feature {NONE} -- Implementation

	is_reversed (entry : MML_PAIR[INTEGER, G]) : BOOLEAN is
			-- Is `entry' a reversed version of an entry in `current'?
		require
			entry_not_void : entry /= Void
		deferred
		ensure
			definition_of_reversed : Result implies (range.is_member (entry.second) and is_defined (count - entry.first + 1))
		end

invariant
	definition_of_sequence: (create {INTEGER_INTERVAL}.make (lower_bound, upper_bound)).for_all (agent domain.anded (?, agent is_defined (?), agent domain.is_member (?)))

	definition_of_sequence_reversal : is_equal (reversed.reversed)
	definition_of_tail : not is_empty implies is_equal (tail.prepended (first))
	definition_of_front : not is_empty implies is_equal (front.appended (last))

	definition_of_count : count.is_equal (domain.cardinality)
	definition_of_index : not is_empty implies lower_bound <= upper_bound

end -- class MML_SEQUENCE