deferred class SPECIFICATION_ACTIVE [G]

inherit
	SPECIFICATION_BAG [G]
		rename
			bag as bag_model
		end

feature -- Access

	item: G is
			-- Current item
		require
			must_be_readable: readable
		deferred
		ensure
			definition_of_item: bag_model.is_member (Result)
			pure_feature: is_equal (old Current)
			immutable_model: bag_model.is_equal (old bag_model)
			active_has_result: has (Result)
			result_not_void: Result /= Void
		end

feature -- Status report

	readable: BOOLEAN is
			-- Is there a current item that may be read?
		deferred
		ensure
			definition_of_readable: Result implies not bag_model.is_empty
			pure_feature: is_equal (old Current)
			immutable_model: bag_model.is_equal (old bag_model)
			result_implies_not_empty: Result implies not is_empty
		end

	writable: BOOLEAN is
			-- Is there a current item that may be modified?
		deferred
		ensure
			definition_of_writable: Result implies not bag_model.is_empty
			pure_feature: is_equal (old Current)
			immutable_model: bag_model.is_equal (old bag_model)
			result_implies_readable: Result implies readable
			result_implies_not_empty: Result implies not is_empty
		end

feature -- Element change

	replace (other: G) is
			-- Replace current item by `other'.
		require
			must_be_writable: writable
			other_not_void: other /= Void
		deferred
		ensure
			definition_of_replace: bag_model.is_equal (old bag_model.domain_element_anti_restricted (item).extended (other))
			immutable_argument: other.is_equal (old other)
			item_was_changed: not item.is_equal (other) implies not is_equal (old Current)
			other_is_member: has (other)
			item_was_replaced: item.is_equal (other)
		end

feature -- Removal

	remove is
			-- Remove current item.
		require
			must_be_prunable: prunable
			must_be_writable: writable
		deferred
		ensure
			definition_of_remove: bag_model.is_equal (old bag_model.pruned (item))
		end

invariant
	writable_implies_readable: writable implies readable
	empty_implies_not_readable_neither_writable: bag_model.is_empty implies (not readable) and (not writable)

end -- class SPECIFICATION_ACTIVE