deferred class SPECIFICATION_ARRAY [G]

inherit
	SPECIFICATION_RESIZABLE [G]

	SPECIFICATION_INDEXABLE [G, INTEGER]

	SPECIFICATION_TO_SPECIAL [G]
		export
			{SPECIFICATION_ARRAY}
				set_area
		end

feature -- Access

	area: SPECIAL [G] is
			-- Special data zone
		deferred
		ensure then
			immutable_models: bag_model.is_equal (old bag_model) and indexable_model.is_equal (old indexable_model)
		end

	item, infix "@" (i: INTEGER): G is
			-- Entry at index `i', if in index interval
		deferred
		ensure then
			definition_of_item: Result.is_equal (area.item (i - lower))
		end

	has (other: G): BOOLEAN is
			-- Does `other' appear in array?
		deferred
		ensure then
			definition_of_has: Result.is_equal (sequence_model.is_member (other))
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model)
		end

feature -- Measurement

	lower: INTEGER
			-- Minimum index

	upper: INTEGER
			-- Maximum index

	additional_space: INTEGER is
			-- Proposed number of additional items.
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
		end

	count, capacity: INTEGER is
			-- Number of available indices
		deferred
		ensure then
			definition_of_count: Result.is_equal (sequence_model.count) and Result.is_equal (indexable_model.cardinality) and Result.is_equal (upper - lower + 1)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
		end

	occurrences (other: G): INTEGER is
			-- Number of times `other' appears in structure
		deferred
		ensure then
			definition_of_occurrences: Result.is_equal (sequence_model.occurrences (other))
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model)
		end

	index_set: SPECIFICATION_INTEGER_INTERVAL is
			-- Range of acceptable indexes
		deferred
		ensure then
			definition_of_index_set: Result.count.is_equal (sequence_model.count) and Result.lower.is_equal (lower) and Result.upper.is_equal (upper)
			immutable_model: sequence_model.is_equal (old sequence_model)
		end

feature -- Status report

	changeable_comparison_criterion: BOOLEAN is
			-- May `object_comparison' be changed?
		deferred
		ensure then
			immutable_model: sequence_model.is_equal (old sequence_model)
		end

	full: BOOLEAN is
			-- Is structure full?
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
		end

	is_empty: BOOLEAN is
			-- Is there no element?
		deferred
		ensure then
			definition_of_is_empty: Result.is_equal (sequence_model.is_empty)
			immutable_model: sequence_model.is_equal (old sequence_model)
		end

	object_comparison: BOOLEAN is
			-- Must search operations use `equal' rather than `='
			-- for comparing references? (Default: no, use `='.)
		deferred
		ensure then
			immutable_model: sequence_model.is_equal (old sequence_model)
		end

	resizable: BOOLEAN is
			-- May `capacity' be changed?
		deferred
		ensure then
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
		end

	valid_index (i: INTEGER): BOOLEAN is
			-- Is `i' within the bounds of the array?
		deferred
		ensure then
			definition_of_valid_index: Result.is_equal (lower <= i and i <= upper)
		end

	valid_index_set: BOOLEAN is
			-- Is the index set valid?
		deferred
		ensure
			definition_of_valid_index_set: Result.is_equal (index_set.count.is_equal (sequence_model.count))
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
		end

feature -- Element change

	enter (other: G; i: INTEGER) is
			-- Replace `i'-th entry, if in index interval, by `other'.
		deferred
		ensure
			definition_of_enter: sequence_model.is_defined (i) implies sequence_model.is_equal (old sequence_model.replaced (other, i))
			bag_model: sequence_model.is_defined (i) implies bag_model.is_equal (old bag_model.pruned (sequence_model.element (i)).extended (other))
			immutable_model: sequence_model.is_defined (i) implies indexable_model.is_equal (old indexable_model.domain_element_anti_restricted (i).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (i, other)))
		end

	put (other: G; i: INTEGER) is
			-- Replace `i'-th entry, if in index interval, by `other'.
		deferred
		ensure then
			definition_of_put: sequence_model.is_defined (i) implies sequence_model.is_equal (old sequence_model.replaced (other, i))
			bag_model: sequence_model.is_defined (i) implies bag_model.is_equal (old bag_model.pruned (sequence_model.element (i)).extended (other))
			immutable_model: sequence_model.is_defined (i) implies indexable_model.is_equal (old indexable_model.domain_element_anti_restricted (i).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (i, other)))
		end

invariant
	area_not_void: area /= Void

	definition_of_capacity: capacity = upper - lower + 1
	definition_of_index_set: valid_index_set

	non_negative_count: count >= 0

end -- class SPECIFICATION_ARRAY