deferred class SPECIFICATION_CHAIN [G]

inherit
	SPECIFICATION_CURSOR_STRUCTURE [G]
		select
			put
		end

	SPECIFICATION_INDEXABLE [G, INTEGER]
		rename
			item as i_th,
			put as put_i_th,
			negated as indexable_negated,
			ored as indexable_ored,
			anded as indexable_anded,
			xored as indexable_xored,
			implied as indexable_implied,
			curried_adapter as indexable_curried_adapter,
			curried_result_adapter as indexable_curried_result_adapter,
			curried as indexable_curried,
			iterated as indexable_iterated,
			iterated_adapter as indexable_iterated_adapter
		select
			indexable_ored, indexable_anded, indexable_xored,
			indexable_implied, indexable_curried_adapter,
			indexable_curried_result_adapter, indexable_curried,
			indexable_iterated, indexable_iterated_adapter,
			indexable_negated
		end

	SPECIFICATION_SEQUENCE [G]
		rename
			put as sequence_put
		export
			{NONE}
				sequence_put
		undefine
			composed, composed_adapter, iterated, iterated_adapter,
			curried, curried_adapter, curried_result_adapter,
			negated, ored, xored, anded, implied
		select
			index_of, has, occurrences
		end

	SPECIFICATION_SEQUENCE [G]
		rename
			put as sequence_put,
			index_of as sequential_index_of,
			has as sequential_has,
			occurrences as sequential_occurrences
		export
			{NONE}
				sequential_index_of, sequential_has, sequence_put
		undefine
			composed, composed_adapter, iterated, iterated_adapter,
			curried, curried_adapter, curried_result_adapter,
			negated, ored, xored, anded, implied
		end

feature -- Access

	cursor: SPECIFICATION_CURSOR is
			-- Current cursor position
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	first: G is
			-- Item at first position
		require
			not_empty: not sequence_model.is_empty
		deferred
		ensure
			definition_of_first: Result.is_equal (sequence_model.first)
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and bag_model.is_equal (old bag_model)
		end

	last: G is
			-- Item at last position
		require
			not_empty: not sequence_model.is_empty
		deferred
		ensure
			definition_of_last: Result.is_equal (sequence_model.last)
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and bag_model.is_equal (old bag_model)
		end

	i_th, infix "@" (key: INTEGER): G is
			-- Item at `key'-th position
		deferred
		ensure then
			definition_of_i_th: Result.is_equal (sequence_model.element (key))
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	index: INTEGER is
			-- Index of current position
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	index_of (other: G; i: INTEGER): INTEGER is
			-- Index of `i'-th occurrence of `other'.
			-- 0 if none.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	item: G is
			-- Item at current position
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	sequential_occurrences (other: G): INTEGER is
			-- Number of times `other' appears.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	occurrences (other: G): INTEGER is
			-- Number of times `other' appears.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	search (other: G) is
			-- Move to first position (at or after current
			-- position) where `item' and `other' are equal.
			-- If no such position ensure that `exhausted' will be true.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	exhausted: BOOLEAN is
			-- Has structure been completely explored?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	full: BOOLEAN is
			-- Is structure filled to capacity?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

feature -- Measurement

	count: INTEGER is
			-- Number of items
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	index_set: SPECIFICATION_INTEGER_INTERVAL is
			-- Range of acceptable indexes
		deferred
		ensure then
			definition_of_index_set: Result.set_model.is_equal (sequence_model.domain)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Cursor movement

	back is
			-- Move to previous position.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	before: BOOLEAN is
			-- Is there no valid position to the left of current one?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	start is
			-- Move cursor to first position.
			-- (No effect if empty)
		deferred
		ensure then
			definition_of_start: not sequence_model.is_empty implies sequence_model.element (cursor_model).is_equal (sequence_model.first)
			immutable_models: sequence_model.is_equal (old sequence_model) and indexable_model.is_equal (old indexable_model)
		end

	finish is
			-- Move cursor to last position.
			-- (No effect if empty)
		deferred
		ensure then
			definition_of_finish: not sequence_model.is_empty implies sequence_model.element (cursor_model).is_equal (sequence_model.last)
			empty_implies_not_defined: sequence_model.is_empty implies not sequence_model.is_defined (cursor_model)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
			not_empty_implies_not_off: not is_empty implies not off
		end

	forth is
			-- Move to next position; if no next position,
			-- ensure that `exhausted' will be true.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	move (number: INTEGER) is
			-- Move cursor `number' positions. The cursor
			-- may end up `off' if the absolute value of `number'
			-- is too big.
		deferred
		ensure
			definition_of_move: not sequence_model.is_defined (old cursor_model + number) implies not sequence_model.is_defined (cursor_model)
			immutable_models: sequence_model.is_equal (old sequence_model) and indexable_model.is_equal (old indexable_model) and bag_model.is_equal (old bag_model)
			defined_implies_moved: sequence_model.is_defined (old cursor_model + number) implies cursor_model.is_equal (old cursor_model + number)
			too_far_right_implies_after: (old cursor_model + number > sequence_model.upper_bound) implies cursor_model.is_equal (sequence_model.upper_bound + 1)
			too_far_left_implies_before: (old cursor_model + number < sequence_model.lower_bound) implies cursor_model.is_equal (sequence_model.lower_bound - 1)
		end

	go_i_th (number: INTEGER) is
			-- Move cursor to `number'-th position.
		require
			valid_cursor_index: valid_cursor_index (number)
		deferred
		ensure
			definition_of_go_i_th: cursor_model.is_equal (number)
			immutable_models: sequence_model.is_equal (old sequence_model) and indexable_model.is_equal (old indexable_model) and bag_model.is_equal (old bag_model)
		end

	go_to (other: SPECIFICATION_CURSOR) is
			-- Move cursor to position `other'.
		deferred
		ensure then
			immutable_models: sequence_model.is_equal (old sequence_model) and indexable_model.is_equal (old indexable_model)
		end

 feature -- Status report

	readable: BOOLEAN is
			-- Is there a current item that may be read?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	writable: BOOLEAN is
			-- Is there a current item that may be modified?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	after: BOOLEAN is
			-- Is there no valid position to the right of current one?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	valid_index (number: INTEGER): BOOLEAN is
			-- Is `number' within allowable bounds?
		deferred
		ensure then
			definition_of_valid_index: Result.is_equal (number >= sequence_model.lower_bound and number <= sequence_model.upper_bound)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	isfirst: BOOLEAN is
			-- Is cursor at first position?
		deferred
		ensure
			definition_of_first: Result.is_equal (not sequence_model.is_empty and cursor_model.is_equal (sequence_model.lower_bound))
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	islast: BOOLEAN is
			-- Is cursor at last position?
		deferred
		ensure
			definition_of_last: Result.is_equal (not sequence_model.is_empty and cursor_model.is_equal (sequence_model.upper_bound))
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	off: BOOLEAN is
			-- Is there no current item?
		deferred
		ensure then
			definition_of_off: Result.is_equal (cursor_model.is_equal (sequence_model.lower_bound) or cursor_model.is_equal (sequence_model.upper_bound))
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	valid_cursor_index (number: INTEGER): BOOLEAN is
			-- Is `number' correctly bounded for cursor movement?
		deferred
		ensure
			definition_of_valid_cursor_index: Result.is_equal (number >= sequence_model.lower_bound - 1 and number <= sequence_model.upper_bound + 1)
			pure_feature: is_equal (old Current)
			immutable_argument: number.is_equal (old number)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	valid_cursor (other: SPECIFICATION_CURSOR): BOOLEAN is
			-- Can the cursor be moved to position `other'?
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Element change

	append (other: SPECIFICATION_SEQUENCE [G]) is
			-- Append a copy of `other'.
		deferred
		ensure then
	 		definition_of_extend: indexable_model.cardinality.is_equal (old indexable_model.cardinality + other.sequence_model.count)
		end

	extend (other: G) is
			-- Add a new occurrence of `other'.
		deferred
		ensure then
	 		definition_of_extend: indexable_model.cardinality.is_equal (old indexable_model.cardinality + 1)
	 		immutable_model: cursor_model.is_equal (old cursor_model)
		end

	force (other: G) is
			-- Add `other' to end.
		deferred
		ensure then
	 		definition_of_force: indexable_model.is_equal (old indexable_model.extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (sequence_model.count + 1, other)))
		end

	put (other: G) is
			-- Replace current item by `other'.
		deferred
		ensure then
	 		definition_of_put: sequence_model.is_equal (old sequence_model.replaced (other, cursor_model))
	 		immutable_model: cursor_model.is_equal (old cursor_model)
		end

	put_i_th (other: G; number: INTEGER) is
			-- Put `other' at `number'-th position.
		deferred
		ensure then
			definition_of_put: sequence_model.is_equal (old sequence_model.replaced (other, number))
			immutable_models: cursor_model.is_equal (old cursor_model)
		end

	replace (other: G) is
			-- Replace current item by `other'.
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model.domain_element_anti_restricted (cursor_model).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (cursor_model, other)))
		end

feature -- Removal

feature -- Transformation

	swap (number: INTEGER) is
			-- Exchange item at `number'-th position with item
			-- at cursor position.
		require
			defined_cursor: sequence_model.is_defined (cursor_model)
			defined_index: sequence_model.is_defined (number)
		deferred
		ensure
			definition_of_swap : sequence_model.is_equal (old sequence_model.replaced (sequence_model.element (cursor_model), index).replaced (sequence_model.element (index), cursor_model))
			definition_of_swap: indexable_model.is_equal (old indexable_model.domain_element_anti_restricted (cursor_model).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (cursor_model, sequence_model.element (cursor_model))).domain_element_anti_restricted (index).extended (create {MML_DEFAULT_PAIR[INTEGER, G]}.make_from (cursor_model, sequence_model.element (index))))
			immutable_models: cursor_model.is_equal (old cursor_model) and bag_model.is_equal (old bag_model)
			immutable_argument : index.is_equal (old index)
		end

feature -- Conversion

	linear_representation: SPECIFICATION_LINEAR [G] is
			-- Representation as a linear structure
		deferred
		ensure then
			definition_of_linear_representation: Result.sequence_model.is_equal (sequence_model)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and bag_model.is_equal (old bag_model)
		end

feature -- Duplication

	duplicate (number: INTEGER): like Current is
			-- Copy of sub-chain beginning at current position
			-- and having min (`number', `from_here') items,
			-- where `from_here' is the number of items
			-- at or to the right of current position.
		require
			not_defined_implies_after: not sequence_model.is_defined (cursor_model) implies cursor_model.is_equal (sequence_model.upper_bound + 1)
			non_negative_number: number >= 0
		deferred
		ensure
			definition_of_duplicate: cursor_model + number - 1 <= sequence_model.upper_bound implies Result.sequence_model.is_equal (sequence_model.interval (cursor_model, cursor_model + number - 1))
			definition_of_duplicate: cursor_model + number - 1 > sequence_model.upper_bound implies Result.sequence_model.is_equal (sequence_model.interval (cursor_model, sequence_model.upper_bound))
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and bag_model.is_equal (old bag_model)
			immutable_argument: number.is_equal (old number)
		end

feature -- Iteration

	for_all (test: FUNCTION [ANY, TUPLE [G], BOOLEAN]): BOOLEAN is
			-- Is `test' true for all items?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

	there_exists (test: FUNCTION [ANY, TUPLE [G], BOOLEAN]): BOOLEAN is
			-- Is `test' true for at least one item?
		deferred
		ensure then
			immutable_model: indexable_model.is_equal (old indexable_model)
		end

invariant
	definition_of_index: index.is_equal (cursor_model)
	definition_of_off : off.is_equal (not sequence_model.is_defined (index) and (index < sequence_model.lower_bound or index > sequence_model.upper_bound))

	definition_of_isfirst : isfirst.is_equal (not sequence_model.is_empty and index.is_equal (sequence_model.lower_bound))
	definition_of_islast : islast.is_equal (not sequence_model.is_empty and index.is_equal (sequence_model.upper_bound))

end -- class SPECIFICATION_CHAIN