deferred class SPECIFICATION_COLLECTION [G]

inherit
	SPECIFICATION_CONTAINER [G]

feature -- Status Report

	extendible: BOOLEAN is
			-- May new items be added?
		deferred
		ensure
			pure_feature: is_equal (old Current)
		end

	prunable: BOOLEAN is
			-- May items be removed?
		deferred
		ensure
			pure_feature: is_equal (old Current)
		end

	is_inserted (other: G): BOOLEAN is
			-- Has `other' been inserted by the most recent insertion?
		require
			other_not_void: other /= Void
		deferred
		ensure
			result_implies_has: Result implies has (other)
			result_implies_extendible: Result implies (old extendible)
			pure_feature: is_equal (old Current)
			immutable_argument: other.is_equal (old other)
		end

feature -- Element Change

	extend, put (other: G) is
			-- Ensure that structure includes `other'.
		require
			other_not_void: other /= Void
			must_be_extendible: extendible
		deferred
		ensure
			item_is_inserted: is_inserted (other)
			has_now_other: has (other)
			not_empty_anymore: not is_empty
			immutable_argument: other.is_equal (old other)
		end

	fill (other: SPECIFICATION_CONTAINER [G]) is
			-- Fill with as many items of `other' as possible.
			-- The representations of `other' and current structure
			-- need not be the same.
		require
			other_not_void: other /= Void
			must_be_extendible: extendible
		deferred
		ensure
			extendible_implies_filled_in: (old extendible) and not other.linear_representation.is_empty implies not is_empty
			not_extendible_implies_untouched: not (old extendible) implies is_equal (old Current)
			immutable_argument: other.is_equal (old other)
		end

feature -- Removal

	prune (other: G) is
			-- Remove one occurrence of `other' if any.
		require
			other_not_void: other /= Void
			must_be_prunable: prunable
		deferred
		ensure
			has_implies_pruned: old (has (other)) implies not is_equal (old Current)
			immutable_argument: other.is_equal (old other)
		end

	prune_all (other: G) is
			-- Remove all occurrences of `other'.
		require
			other_not_void: other /= Void
			must_be_prunable: prunable
		deferred
		ensure
			immutable_argument: other.is_equal (old other)
			has_implies_pruned: not old (has (other)) implies is_equal (old Current)
			has_not_other: not has (other)
		end

	wipe_out is
			-- Remove all items.
		require
			must_be_prunable: prunable
		deferred
		ensure
			definition_of_wipe_out: is_empty
		end

end -- class SPECIFICATION_COLLECTION