deferred class
	SPECIFICATION_CONTAINER [G]

inherit
	SPECIFICATION_ANY

feature -- Access

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		require
			other_not_void: other /= Void
		deferred
		ensure
			not_found_in_empty: Result implies not is_empty
			pure_feature: is_equal (old Current)
			immutable_argument: other.is_equal (old other)
		end

feature -- Status report

	is_empty: BOOLEAN is
			-- Is there no element?
		deferred
		ensure
			pure_feature: is_equal (old Current)
		end

	object_comparison: BOOLEAN is
			-- Must search operations use `equal' rather than `='
			-- for comparing references? (Default: no, use `='.)
		deferred
		ensure
			pure_feature: is_equal (old Current)
		end

	changeable_comparison_criterion: BOOLEAN is
			-- May `object_comparison' be changed?
		deferred
		ensure
			pure_feature: is_equal (old Current)
		end

feature -- Status Setting

	compare_objects is
			-- Ensure that future search operations will use `equal'
			-- rather than `=' for comparing references.
		require
			changeable_comparison_criterion: changeable_comparison_criterion
		deferred
		ensure
			object_comparison_set: object_comparison
		end

	compare_references is
			-- Ensure that future search operations will use `='
			-- rather than `equal' for comparing references.
		require
			changeable_comparison_criterion: changeable_comparison_criterion
		deferred
		ensure
			reference_comparison_set: not object_comparison
		end

feature -- Conversion

	linear_representation: SPECIFICATION_LINEAR [G] is
			-- Representation as a linear structure
		deferred
		ensure
			result_in_current: Result.for_all (agent has (?))
			empty_result_equals_empty: Result.is_empty.is_equal (is_empty)
			pure_feature: is_equal (old Current)
			result_not_void : Result /= Void
		end

invariant
	definition_of_linear_representation: linear_representation.for_all (agent has (?))
	empty_linear_equals_empty: linear_representation.is_empty.is_equal (is_empty)

end -- class SPECIFICATION_CONTAINER