deferred class SPECIFICATION_DYNAMIC_CHAIN [G]

inherit
	SPECIFICATION_CHAIN [G]
		export
			{ANY} remove, prune_all, prune
		end

	SPECIFICATION_UNBOUNDED [G]

feature -- Status report

	prunable: BOOLEAN is
			-- May items be removed?
		deferred
		ensure then
			definition_of_prunable: Result.is_equal (True)
		end

feature -- Element change

	put_front (other: like item) is
			-- Add `other' at beginning.
			-- Do not move cursor.
		require
			other_not_void: other /= Void
		deferred
		ensure
			definition_of_put_front: sequence_model.is_equal (old sequence_model.prepended (other)) and (old cursor_model >= sequence_model.lower_bound) implies cursor_model.is_equal (old cursor_model + 1)
			definition_of_put_front: sequence_model.first.is_equal (other) and (old cursor_model < sequence_model.lower_bound) implies cursor_model.is_equal (old cursor_model)
			immutable_argument: other.is_equal (old other)
			bag_model_extended: bag_model.is_equal (old bag_model.extended (other))
			indexable_model_cardinality: indexable_model.cardinality.is_equal (old indexable_model.cardinality + 1)
			remains_undefined: (old not sequence_model.is_defined (cursor_model)) implies not sequence_model.is_defined (cursor_model)
		end

	put_left (other: like item) is
			-- Add `other' to the left of cursor position.
			-- Do not move cursor.
		require
			must_be_extendible: extendible
			defined_cursor: cursor_model >= sequence_model.lower_bound
		deferred
		ensure
			definition_of_put_front: (old cursor_model.is_equal (sequence_model.lower_bound)) implies sequence_model.is_equal (old sequence_model.prepended (other))
			definition_of_put_tail: (old cursor_model > sequence_model.upper_bound) implies sequence_model.is_equal (old sequence_model.appended (other))
			definition_of_put_middle: (old cursor_model > sequence_model.lower_bound and cursor_model <= sequence_model.upper_bound) implies sequence_model.is_equal (old sequence_model.interval (sequence_model.lower_bound, cursor_model - 1).appended (other).concatenated (sequence_model.interval (cursor_model, sequence_model.upper_bound)))
			immutable_argument: other.is_equal (old other)
			cursor_model_increased: cursor_model.is_equal (old cursor_model + 1)
			indexable_model_cardinality: indexable_model.cardinality.is_equal (old indexable_model.cardinality + 1)
			bag_model_extended: bag_model.is_equal (old bag_model.extended (other))
		end

	put_right (other: like item) is
			-- Add `other' to the right of cursor position.
			-- Do not move cursor.
		require
			must_be_extendible: extendible
			defined_cursor: cursor_model <= sequence_model.upper_bound
		deferred
		ensure
			definition_of_put_tail: (old cursor_model.is_equal (sequence_model.upper_bound)) implies sequence_model.is_equal (old sequence_model.appended (other))
			definition_of_put_front: (old cursor_model < sequence_model.lower_bound) implies sequence_model.is_equal (old sequence_model.prepended (other))
			definition_of_put_middle: (old cursor_model >= sequence_model.lower_bound and cursor_model < sequence_model.upper_bound) implies sequence_model.is_equal (old sequence_model.interval (sequence_model.lower_bound, cursor_model).appended (other).concatenated (sequence_model.interval (cursor_model + 1, sequence_model.upper_bound)))
			immutable_argument: other.is_equal (old other)
			bag_model_extended: bag_model.is_equal (old bag_model.extended (other))
			indexable_model_cardinality: indexable_model.cardinality.is_equal (old indexable_model.cardinality + 1)
			immutable_cursor: cursor_model.is_equal (old cursor_model)
		end

	merge_left (other: like Current) is
			-- Merge `other' into current structure before cursor
			-- position. Do not move cursor. Empty `other'.
		require
			must_be_extendible: extendible
			defined_cursor: cursor_model >= sequence_model.lower_bound
			other_not_void: other /= Void
			not_equal_other: not is_equal (other)
		deferred
		ensure
			definition_of_merge_front: (old cursor_model.is_equal (sequence_model.lower_bound)) implies sequence_model.is_equal (old other.sequence_model.concatenated (sequence_model))
			definition_of_merge_tail: (old cursor_model > sequence_model.upper_bound) implies sequence_model.is_equal (old sequence_model.concatenated (other.sequence_model))
			definition_of_merge_middle: (old cursor_model > sequence_model.lower_bound and cursor_model <= sequence_model.upper_bound) implies sequence_model.is_equal (old sequence_model.interval (sequence_model.lower_bound, cursor_model - 1).concatenated (other.sequence_model).concatenated (sequence_model.interval (cursor_model, sequence_model.upper_bound)))
			cursor_model_increased: cursor_model.is_equal (old cursor_model + 1)
			bag_model_extended: bag_model.is_equal (old bag_model.added (other.bag_model))
			indexable_model_cardinality: indexable_model.cardinality.is_equal (old indexable_model.cardinality + other.indexable_model.cardinality)
			other_is_empty: other.sequence_model.is_empty and other.bag_model.is_empty and other.indexable_model.is_empty
		end

	merge_right (other: like Current) is
			-- Merge `other' into current structure after cursor
			-- position. Do not move cursor. Empty `other'.
		require
			must_be_extendible: extendible
			defined_cursor: cursor_model <= sequence_model.upper_bound
			other_not_void: other /= Void
			not_equal_other: not is_equal (other)
		deferred
		ensure
			definition_of_put_tail: (old cursor_model.is_equal (sequence_model.upper_bound)) implies sequence_model.is_equal (old sequence_model.concatenated (other.sequence_model))
			definition_of_put_front: (old cursor_model < sequence_model.lower_bound) implies sequence_model.is_equal (old other.sequence_model.concatenated (sequence_model))
			definition_of_put_middle: (old cursor_model >= sequence_model.lower_bound and cursor_model < sequence_model.upper_bound) implies sequence_model.is_equal (old sequence_model.interval (sequence_model.lower_bound, cursor_model).concatenated (other.sequence_model).concatenated (sequence_model.interval (cursor_model + 1, sequence_model.upper_bound)))
			immutable_cursor: cursor_model.is_equal (old cursor_model)
			bag_model_extended: bag_model.is_equal (old bag_model.added (other.bag_model))
			indexable_model_cardinality: indexable_model.cardinality.is_equal (old indexable_model.cardinality + other.indexable_model.cardinality)
			other_is_empty: other.sequence_model.is_empty and other.bag_model.is_empty and other.indexable_model.is_empty
		end

feature -- Removal

	prune (other: G) is
			-- Remove first occurrence of `other', if any,
			-- after cursor position.
			-- If found, move cursor to right neighbor;
			-- if not, make structure `exhausted'.
		deferred
		ensure then
			occurrence_implies_right_neighbor: (old sequence_model.occurrences (other) > 0) implies cursor_model.is_equal (old sequence_model.indexes_element_ordered (other).first + 1)
			not_found_implies_exhaused: (old sequence_model.occurrences (other).is_equal (0)) implies exhausted
			indexable_model: (old sequence_model.occurrences (other) > 0) implies indexable_model.cardinality.is_equal (old indexable_model.cardinality - 1)
		end

	remove_left is
			-- Remove item to the left of cursor position.
			-- Do not move cursor.
		require
			defined_cursor: cursor_model > sequence_model.lower_bound and cursor_model <= sequence_model.upper_bound + 1
		deferred
		ensure
			definition_of_remove_left: sequence_model.is_equal (old sequence_model.pruned_at (cursor_model - 1))
			indexable_model: indexable_model.cardinality.is_equal (old indexable_model.cardinality - 1)
			bag_model: bag_model.is_equal (old bag_model.pruned (sequence_model.element (cursor_model)))
			same_cursor: cursor_model.is_equal (old cursor_model - 1)
		end

	remove_right is
			-- Remove item to the right of cursor position.
			-- Do not move cursor.
		require
			defined_cursor: cursor_model < sequence_model.upper_bound and cursor_model >= sequence_model.lower_bound - 1
		deferred
		ensure
			definition_of_remove_right: sequence_model.is_equal (old sequence_model.pruned_at (cursor_model + 1))
			indexable_model: indexable_model.cardinality.is_equal (old indexable_model.cardinality - 1)
			bag_model: bag_model.is_equal (old bag_model.pruned (sequence_model.element (cursor_model)))
			immutable_cursor: cursor_model.is_equal (old cursor_model)
		end

	prune_all (other: G) is
			-- Remove all occurrences of `other'.
			-- Leave structure `exhausted'.
		deferred
		ensure then
			cursor_off: cursor_model.is_equal (sequence_model.upper_bound + 1)
		end

	wipe_out is
			-- Remove all items.
		deferred
		ensure then
			definition_of_wipe_out: sequence_model.is_empty
			no_defined_cursor: not sequence_model.is_defined (cursor_model)
		end

invariant
	definition_of_extendible: extendible

end -- class SPECIFICATION_DYNAMIC_CHAIN