deferred class SPECIFICATION_INTEGER_INTERVAL

inherit
	SPECIFICATION_RESIZABLE [INTEGER]

	SPECIFICATION_INDEXABLE [INTEGER, INTEGER]
		rename
			put as indexable_put
		select
			bag_put
		end

	SPECIFICATION_SET [INTEGER]

	MML_FUNCTIONALS [INTEGER]
		export
			{NONE}
				all
		end

feature -- Initialization

	adapt (other: SPECIFICATION_INTEGER_INTERVAL) is
			-- Reset to be the same interval as `other'.
		require
			other_not_void: other /= Void
		deferred
		ensure
			equals_other: is_equal (other)
			equals_bag: bag_model.is_equal (other.bag_model)
			equals_set: set_model.is_equal (other.set_model)
			equals_indexes: indexable_model.is_equal (other.indexable_model)
			immutable_argument: other.is_equal (old other)
			same_lower_bound: lower.is_equal (other.lower)
			same_upper_bound: upper.is_equal (other.upper)
			same_lower_defined: lower_defined.is_equal (other.lower_defined)
			same_upper_defined: upper_defined.is_equal (other.upper_defined)
		end

feature -- Access

	lower_defined: BOOLEAN
			-- Is there a lower bound?

	lower: INTEGER is
			-- Smallest value in interval
		require
			lower_bound_defined: lower_defined
		deferred
		ensure
			definition_of_lower: indexable_model.image_element (Result).cardinality.is_equal (1)
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
			identity_implies_equal_element: indexable_model.image_element (Result).any_element.is_equal (Result)
		end

	upper_defined: BOOLEAN
			-- Is there an upper bound?

	upper: INTEGER is
			-- Largest value in interval
		require
			upper_bound_defined: upper_defined
		deferred
		ensure
			definition_of_upper: indexable_model.image_element (Result).cardinality.is_equal (1)
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
			identity_implies_equal_element: indexable_model.image_element (Result).any_element.is_equal (Result)
		end

	item, infix "@" (i: INTEGER): INTEGER is
			-- Entry at index `i', if in index interval
		deferred
		ensure then
			definition_of_item: indexable_model.domain_element_restricted (i).for_all (agent {MML_PAIR[INTEGER, INTEGER]}.is_identity) and indexable_model.domain_element_restricted (i).cardinality.is_equal (1)
			immutable_model: set_model.is_equal (old set_model)
		end

	has, valid_index (other: INTEGER): BOOLEAN is
			-- Does `other' appear in interval?
		deferred
		ensure then
			definition_of_valid_index: Result.is_equal ((upper_defined implies other <= upper) and (lower_defined implies other >= lower))
			definition_of_valid_index: upper_defined and lower_defined implies Result.is_equal (set_model.is_member (other))
		end

feature -- Measurement

	additional_space: INTEGER is
			-- Proposed number of additional items.
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

	occurrences (other: INTEGER): INTEGER is
			-- Number of times `other' appears in structure
		deferred
		ensure then
			definition_of_occurrences: Result.is_equal (indexable_model.domain_element_restricted (other).cardinality)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model)
			has_implies_occurrence: (Result.is_equal (1) implies has (other)) and (not Result.is_equal (1) implies Result.is_equal (0))
		end

	capacity: INTEGER is
			-- Maximum number of items in interval
			-- (here the same thing as `count')
		require else
			defined_bounds : lower_defined and upper_defined
		deferred
		ensure then
			definition_of_capacity : Result.is_equal (count)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

	count: INTEGER is
			-- Number of items in interval
		require else
			defined_bounds : lower_defined and upper_defined
		deferred
		ensure then
			definition_of_count: (upper_defined and lower_defined) implies Result.is_equal (indexable_model.cardinality)
			immutable_models: indexable_model.is_equal (old indexable_model) and bag_model.is_equal (old bag_model)
			defined_bounds: (upper_defined and lower_defined) implies Result.is_equal (upper - lower + 1) and not (upper_defined and lower_defined) implies Result.is_equal (0)
		end

	index_set: SPECIFICATION_INTEGER_INTERVAL is
			-- Range of acceptable indexes
		deferred
		ensure then
			definition_of_index_set: is_equal (Result)
			immutable_set_model: set_model.is_equal (old set_model)
		end

feature -- Status report

	full: BOOLEAN is
			-- Is structure full?
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

	is_empty: BOOLEAN is
			-- Is there no element?
		deferred
		ensure then
			definition_of_is_empty: Result.is_equal (set_model.is_empty)
			immutable_model: set_model.is_equal (old set_model)
		end

	object_comparison: BOOLEAN is
			-- Must search operations use `equal' rather than `='
			-- for comparing references? (Default: no, use `='.)
		deferred
		ensure then
			immutable_model: set_model.is_equal (old set_model)
		end

	all_cleared: BOOLEAN is
			-- Are all items set to default values?
		deferred
		ensure
			definition_of_all_cleared: Result.is_equal (lower.is_equal (0) and upper.is_equal (0))
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

	extendible: BOOLEAN is
			-- May new items be added?
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model)
		end

	prunable: BOOLEAN is
			-- May individual items be removed?
		deferred
		ensure then
			definition_of_prunable: Result.is_equal (False)
			immutable_model: set_model.is_equal (old set_model)
		end

feature -- Status Setting

	compare_objects is
			-- Ensure that future search operations will use `equal'
			-- rather than `=' for comparing references.
		deferred
		ensure then
			immutable_model: set_model.is_equal (old set_model)
		end

	compare_references is
			-- Ensure that future search operations will use `='
			-- rather than `equal' for comparing references.
		deferred
		ensure then
			immutable_model: set_model.is_equal (old set_model)
		end

feature -- Element change

	extend, put (other: INTEGER) is
			-- Make sure that interval goes all the way
			-- to `other' (up or down).
		deferred
		ensure then
			definition_of_extend: lower.is_equal ((old lower).min (other)) and upper.is_equal ((old upper).max (other))
		end

feature -- Resizing

	automatic_grow is
			-- Change the capacity to accommodate at least
			-- `Growth_percentage' more items.
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

	resize (min_index, max_index: INTEGER) is
			-- Rearrange interval to go from at most
			-- `min_index' to at least `max_index',
			-- encompassing previous bounds.
		require
			valid_indexes: max_index >= min_index
		deferred
		ensure
			definition_of_resize: lower.is_equal (min_index.min (lower)) and upper.is_equal (max_index.max (upper))
			immutable_arguments: min_index.is_equal (old min_index) and max_index.is_equal (old max_index)
			correct_cardinality: set_model.cardinality >= (max_index - min_index + 1) and indexable_model.cardinality >= (max_index - min_index + 1)
		end

	resize_exactly (min_index, max_index: INTEGER) is
			-- Rearrange interval to go from
			-- `min_index' to `max_index'.
		require
			valid_indexes: max_index >= min_index
		deferred
		ensure
			definition_of_resize_exactly: lower.is_equal (min_index) and upper.is_equal (max_index)
			immutable_arguments: min_index.is_equal (old min_index) and max_index.is_equal (old max_index)
			correct_cardinality: set_model.cardinality.is_equal (max_index - min_index + 1) and indexable_model.cardinality.is_equal (max_index - min_index + 1)
		end

	grow (i: INTEGER) is
			-- Ensure that capacity is at least `i'.
		deferred
		ensure then
			bigger_capacity_implies_unchanged: capacity >= i implies is_equal (old Current) and set_model.is_equal (old set_model) and indexable_model.is_equal (old indexable_model)
			less_capacity_implies_resize: capacity < i implies lower.is_equal (old lower) and upper.is_equal (lower + i - 1) and set_model.cardinality >=  i and indexable_model.cardinality >= i
			no_interval_loss: lower <= old lower and upper >= old upper
		end

feature -- Removal

	prune_all (other: INTEGER) is
			-- Remove all occurrences of `other'.
		deferred
		ensure then
			definition_of_prune_all: set_model.is_equal (old set_model.pruned (other))
		end

	wipe_out is
			-- Remove all items.
		deferred
		ensure then
			definition_of_wipe_out: set_model.is_empty
		end

feature -- Conversion

	linear_representation: SPECIFICATION_LINEAR [INTEGER] is
			-- Representation as a linear structure
		deferred
		ensure then
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

feature -- Duplication

	subinterval (start_pos, end_pos: INTEGER): like Current is
			-- Interval made of items of current array within
			-- bounds `start_pos' and `end_pos'.
		require
			valid_indexes: start_pos <= end_pos
		deferred
		ensure
			definition_of_subinterval: Result.lower.is_equal (start_pos) and Result.upper.is_equal (end_pos)
			pure_feature: is_equal (old Current)
			immutable_arguments: start_pos.is_equal (old start_pos) and end_pos.is_equal (old end_pos)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
		end

feature -- Iteration

	for_all (condition: FUNCTION [ANY, TUPLE [INTEGER], BOOLEAN]): BOOLEAN is
			-- Do all interval's values satisfy `condition'?
		require
			finite_interval: upper_defined and lower_defined
		deferred
		ensure
			definition_of_for_all: Result.is_equal (hold_count (condition).is_equal (count))
			pure_feature: is_equal (old Current)
			immutable_argument: condition.is_equal (old condition)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
			for_all_implies_not_exists: Result.is_equal (not exists (agent negated (?, condition)))
		end

	exists (condition: FUNCTION [ANY, TUPLE [INTEGER], BOOLEAN]): BOOLEAN is
			-- Does at least one of  interval's values
			-- satisfy `condition'?
		require
			finite_interval: upper_defined and lower_defined
		deferred
		ensure
			definition_of_exists: Result.is_equal (hold_count (condition) > 0)
			pure_feature: is_equal (old Current)
			immutable_argument: condition.is_equal (old condition)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
			exists_implies_not_for_all: Result.is_equal (not for_all (agent negated (?, condition)))
		end

	exists1 (condition: FUNCTION [ANY, TUPLE [INTEGER], BOOLEAN]): BOOLEAN is
			-- Does exactly one of  interval's values
			-- satisfy `condition'?
		require
			finite_interval: upper_defined and lower_defined
		deferred
		ensure
			definition_of_exists: Result.is_equal (hold_count (condition).is_equal (1))
			pure_feature: is_equal (old Current)
			immutable_argument: condition.is_equal (old condition)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
			exists_implies_not_for_all: Result implies not for_all (agent negated (?, condition))
		end

	hold_count (condition: FUNCTION [ANY, TUPLE [INTEGER], BOOLEAN]): INTEGER is
			-- Number of  interval's values that
			-- satisfy `condition'
		require
			finite_interval: upper_defined and lower_defined
		deferred
		ensure
			definition_of_hold_count: Result >= 0
			pure_feature: is_equal (old Current)
			immutable_argument: condition.is_equal (old condition)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model) and bag_model.is_equal (old bag_model)
			positive_result_implies_exists: Result > 0 implies exists (condition)
		end

feature {NONE} -- Inapplicable

	prune (other: INTEGER) is
			-- Remove one occurrence of `other' if any.
			-- Not applicable here.
		deferred
		ensure then
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model)
		end

	indexable_put (other: INTEGER; k: INTEGER) is
			-- Associate value `other' with key `k'.
			-- Not applicable here.
		deferred
		ensure then
			pure_feature: is_equal (old Current)
			immutable_models: indexable_model.is_equal (old indexable_model) and set_model.is_equal (old set_model)
		end

invariant
	definition_of_count: (upper_defined and lower_defined) implies count.is_equal (upper - lower + 1)
	definition_of_index_set: equal (index_set, Current)

	definition_of_indexable_model: indexable_model.is_identity

	interval_not_infinite: upper_defined and lower_defined
	glueing_invariant: indexable_model.domain.is_equal (set_model)

end -- class INTEGER_INTERVAL