deferred class SPECIFICATION_LINEAR [G]

inherit
	SPECIFICATION_TRAVERSABLE [G]

feature -- Access

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			not_exhausted_implies_result: not exhausted implies Result
		end

	index_of (other: like item; i: INTEGER): INTEGER is
			-- Index of `i'-th occurrence of `other'.
			-- 0 if none.
		require
			other_not_void: other /= Void
			positive_occurrences: i > 0
		deferred
		ensure
			definition_of_index: (Result > 0) implies Result.is_equal (sequence_model.indexes_element_ordered (other).element (i)) and (Result.is_equal (0) implies not sequence_model.is_member (other))
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			immutable_other: other.is_equal (old other)
			immutable_index: i.is_equal (old i)
			non_negative_result: Result >= 0
		end

	search (other: like item) is
			-- Move to first position (at or after current
			-- position) where `item' and `other' are equal.
			-- If no such position ensure that `exhausted' will be true.
		require
			other_not_void: other /= Void
		deferred
		ensure
			definition_of_search : (sequence_model.is_member (other) implies sequence_model.indexes_element_ordered (other).element (sequence_model.lower_bound).is_equal (cursor_model)) and (not sequence_model.is_member (other) implies not sequence_model.is_defined (cursor_model))
			immutable_model: sequence_model.is_equal (old sequence_model)
			immutable_argument: other.is_equal (other)
			exhausted_implies_not_member: exhausted implies not sequence_model.is_member (other)
			not_exhausted_and_object_comparison_implies_found: (not exhausted and object_comparison and other /= Void) implies other.is_equal (item)
			not_exhausted_and_not_object_comparison_implies_found: (not exhausted and object_comparison) implies other.is_equal (item)
			off_implies_exhausted: off implies exhausted
		end

	index: INTEGER is
			-- Index of current position
		deferred
		ensure
			definition_of_index: Result.is_equal (cursor_model)
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			result_zero_implies_off: Result.is_equal (0) implies off
			non_negative_result: Result >= 0
		end

	occurrences (other: G): INTEGER is
			-- Number of times `other' appears.
		require
			other_not_void: other /= Void
		deferred
		ensure
			definition_of_occurrences : Result.is_equal (sequence_model.range_element_restricted (other).count)
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			immutable_argument: other.is_equal (old other)
			non_zero_result_implies_has: Result > 0 implies has (other)
			non_negative_result: Result >= 0
		end

feature -- Status report

	exhausted: BOOLEAN is
			-- Has structure been completely explored?
		deferred
		ensure
			definition_of_exhausted: Result implies cursor_model.is_equal (sequence_model.upper_bound + 1)
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			off_implies_result: off implies Result
		end

	after: BOOLEAN is
			-- Is there no valid position to the right of current one?
		deferred
		ensure
			definition_of_after: Result.is_equal (not sequence_model.is_defined (cursor_model) and cursor_model.is_equal (sequence_model.upper_bound + 1))
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			result_implies_off: Result implies off
		end

	off: BOOLEAN is
			-- Is there no current item?
		deferred
		ensure then
			after_implies_off: after implies off
		end

feature -- Cursor Movement

	forth is
			-- Move to next position; if no next position,
			-- ensure that `exhausted' will be true.
		require
			not_after: not after
			defined_cursor: cursor_model <= sequence_model.upper_bound
		deferred
		ensure
			definition_of_forth: cursor_model.is_equal (old cursor_model + 1)
			immutable_model: sequence_model.is_equal (old sequence_model)
			not_defined_implies_exhausted: not sequence_model.is_defined (cursor_model) implies exhausted
			not_outside_implies_moved: not (old off) and not (old after) implies index.is_equal (old index + 1)
			off_implies_exhausted: off implies exhausted
		end

invariant
	definition_of_off: off implies not sequence_model.is_defined (cursor_model)
	definition_of_index: index.is_equal (cursor_model)

	after_implies_off: after implies off

end -- class SPECIFICATION_LINEAR