deferred class SPECIFICATION_SEQUENCE [G]

inherit
	SPECIFICATION_ACTIVE [G]

	SPECIFICATION_BILINEAR [G]

	SPECIFICATION_FINITE [G]

feature -- Access

	index: INTEGER is
			-- Index of current position
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	index_of (other: G; i: INTEGER): INTEGER is
			-- Index of `i'-th occurrence of `other'.
			-- 0 if none.
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	off: BOOLEAN is
			-- Is there no current item?
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	search (other: G) is
			-- Move to first position (at or after current
			-- position) where `item' and `other' are equal.
			-- If no such position ensure that `exhausted' will be true.
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

feature -- Measurement

	count: INTEGER is
			-- Number of items
		deferred
		ensure then
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Status report

	after: BOOLEAN is
			-- Is there no valid position to the right of current one?
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	exhausted: BOOLEAN is
			-- Has structure been completely explored?
			-- (from SPECIFICATION_LINEAR)
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	full: BOOLEAN is
			-- Is structure filled to capacity?
		deferred
		ensure then
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model) and cursor_model.is_equal (old cursor_model)
		end

	readable: BOOLEAN is
			-- Is there a current item that may be read?
		deferred
		ensure then
			definition_of_readable: Result.is_equal (sequence_model.is_defined (cursor_model))
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			not_off_implies_result: not off implies Result
		end

	writable: BOOLEAN is
			-- Is there a current item that may be modified?
		deferred
		ensure then
			definition_of_readable: Result.is_equal (sequence_model.is_defined (cursor_model))
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			not_off_implies_result: not off implies Result
		end

	extendible: BOOLEAN is
			-- May new items be added?)
		deferred
		ensure then
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model) and cursor_model.is_equal (old cursor_model)
		end

	is_inserted (other: G): BOOLEAN is
			-- Has `other' been inserted by the most recent insertion?
		deferred
		ensure then
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model) and cursor_model.is_equal (old cursor_model)
		end

	prunable: BOOLEAN is
			-- May items be removed?
		deferred
		ensure then
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Cursor movement

	back is
			-- Move to previous position.
		deferred
		ensure then
			definition_of_back: cursor_model >= sequence_model.lower_bound implies cursor_model.is_equal (old cursor_model - 1)
			definition_of_back: cursor_model < sequence_model.lower_bound implies cursor_model.is_equal (old cursor_model)
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model)
		end

	before: BOOLEAN is
			-- Is there no valid position to the left of current one?
		deferred
		ensure then
			definition_of_before: Result.is_equal (cursor_model.is_equal (sequence_model.lower_bound))
			immutable_models: sequence_model.is_equal (old sequence_model) and bag_model.is_equal (old bag_model) and cursor_model.is_equal (old cursor_model)
		end

	forth is
			-- Move to next position; if no next position,
			-- ensure that `exhausted' will be true.
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	start is
			-- Move to first position if any.
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

feature -- Element change

	force (other: G) is
			-- Add `other' to end.
		require
			other_not_void: other /= Void
			must_be_extendible: extendible
		deferred
		ensure
			definition_of_force: sequence_model.is_equal (old sequence_model.appended (other))
			definition_of_force: bag_model.is_equal (old bag_model.extended (other))
			immutable_argument: other.is_equal (old other)
			one_more_occurrence: bag_model.multiplicity (other).is_equal (old bag_model.multiplicity (other) + 1)
			after_implies_increased: old cursor_model.is_equal (sequence_model.upper_bound + 1) implies cursor_model.is_equal (old cursor_model + 1)
			middle_implies_immutable: old cursor_model <= old sequence_model.upper_bound implies cursor_model.is_equal (old cursor_model)
		end

	extend (other: G) is
			-- Add a new occurrence of `other'.
		deferred
		ensure then
			definition_of_extend: sequence_model.range.is_equal (old sequence_model.range.extended (other))
		end

	append (other: SPECIFICATION_SEQUENCE [G]) is
			-- Append a copy of `other'.
		require
			other_not_void: other /= Void
		deferred
		ensure
			definition_of_append: sequence_model.is_equal (old sequence_model.concatenated (other.sequence_model))
			definition_of_append: bag_model.is_subbag (old bag_model.united (other.bag_model))
			immutable_argument: other.is_equal (old other)
			immutable_cursor: cursor_model.is_equal (old cursor_model)
		end

	put (other: G) is
			-- Add `other' to end.
		deferred
		ensure then
			definition_of_put: sequence_model.is_equal (old sequence_model.appended (other))
			definition_of_put: bag_model.is_equal (old bag_model.extended (other))
			one_more_occurrence: bag_model.multiplicity (other).is_equal (old bag_model.multiplicity (other) + 1)
			after_implies_increased: old cursor_model.is_equal (sequence_model.upper_bound + 1) implies cursor_model.is_equal (old cursor_model + 1)
			middle_implies_immutable: old cursor_model <= old sequence_model.upper_bound implies cursor_model.is_equal (old cursor_model)
		end

	replace (other: G) is
			-- Replace current item by `other'.
		deferred
		ensure then
			definition_of_replace: sequence_model.is_equal (old sequence_model.replaced (other, cursor_model))
			immutable_model: cursor_model.is_equal (old cursor_model)
		end

feature -- Removal

	prune (other: G) is
			-- Remove the first occurrence of `other' if any.
			-- If no such occurrence go `off'.
		deferred
		ensure then
			definition_of_prune: old sequence_model.occurrences (other) > 0 implies sequence_model.is_equal (old sequence_model.pruned_at (sequence_model.indexes_element_ordered (other).first))
			definition_of_prune: old sequence_model.occurrences (other) > 0 implies (cursor_model >= sequence_model.indexes_element_ordered (other).first implies cursor_model.is_equal (old cursor_model - 1))
			none_implies_off: old sequence_model.occurrences (other).is_equal (0) implies cursor_model.is_equal (sequence_model.upper_bound + 1)
		end

	prune_all (other: G) is
			-- Remove all occurrences of `other'; go `off'.
		deferred
		ensure then
			definition_of_prune_all: sequence_model.is_equal (old sequence_model.pruned (other))
			cursor_is_off: cursor_model.is_equal (sequence_model.upper_bound + 1)
		end

	remove is
			-- Remove current item.
		deferred
		ensure then
			definition_of_remove: sequence_model.is_equal (old sequence_model.pruned_at (cursor_model))
			immutable_model: cursor_model.is_equal (old cursor_model)
		end

	wipe_out is
			-- Remove all items.
		deferred
		ensure then
			definition_of_wipe_out: sequence_model.is_empty
			undefined_cursor: not sequence_model.is_defined (cursor_model)
		end

feature -- Iteration

	for_all (test: FUNCTION [ANY, TUPLE [G], BOOLEAN]): BOOLEAN is
			-- Is `test' true for all items?
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

	there_exists (test: FUNCTION [ANY, TUPLE [G], BOOLEAN]): BOOLEAN is
			-- Is `test' true for at least one item?
		deferred
		ensure then
			immutable_model: bag_model.is_equal (old bag_model)
		end

invariant
	definition_of_readable: readable.is_equal (not sequence_model.is_empty)
	definition_of_writable: writable.is_equal (not sequence_model.is_empty)

	glueing_invariant: bag_model.domain.is_equal (sequence_model.range)

end -- class SPECIFICATION_SEQUENCE