deferred class SPECIFICATION_SET [G]

inherit
	SPECIFICATION_COLLECTION[G]

	MML_SPECIFICATION

feature {MML_SPECIFICATION, ANY} -- Model

	set_model : MML_SET[G] is
			-- The set model
		deferred
		ensure
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		deferred
		ensure then
			result_implies_same_models: Result implies set_model.is_equal (other.set_model)
		end

feature -- Access

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			definition_of_has : Result.is_equal (set_model.is_member (other))
			immutable_model : set_model.is_equal (old set_model)
		end

feature -- Measurement

	count: INTEGER is
			-- Number of items
		deferred
		ensure
			definition_of_count: Result.is_equal (set_model.cardinality)
			pure_feature: is_equal (old Current)
			immutable_model : set_model.is_equal (old set_model)
			result_not_negative: Result >= 0
		end

feature -- Element change

	extend, put (other: G) is
			-- Ensure that set includes `other'.
		deferred
		ensure then
			definition_of_extend: set_model.is_equal ((old set_model).extended (other))
			has_implies_unchanged: old (has (other)) implies (count.is_equal (old count))
			not_has_implies_added: not old has (other) implies (count.is_equal (old count + 1))
		end

feature -- Removal

	prune (other: G) is
			-- Remove `other' if present.
		deferred
		ensure then
			definition_of_prune: set_model.is_equal ((old set_model).pruned (other))
			immutable_argument: other.is_equal (old other)
			has_implies_changed: old has (other) implies (count.is_equal (old count - 1))
			not_has_implies_unchanged: not old has (other) implies (count.is_equal (old count))
			not_has: not has (other)
		end

	changeable_comparison_criterion: BOOLEAN is
			-- May `object_comparison' be changed?
		deferred
		ensure then
			definition_of_changeable_comparison_criterion: Result.is_equal (set_model.is_empty)
			immutable_model : set_model.is_equal (old set_model)
		end

invariant
	definition_of_count: set_model.cardinality.is_equal (count)
	non_negative_count: count >= 0

end -- class SPECIFICATION_SET