deferred class SPECIFICATION_SUBSET [G]

inherit
	SPECIFICATION_SET[G]

feature -- Comparison

	is_subset (other: SPECIFICATION_SUBSET[G]): BOOLEAN is
			-- Is current set a subset of `other'?
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_subset: other.set_model.is_subset (set_model) and Result.is_equal (other.set_model.for_all (agent set_model.is_member (?))) and Result.is_equal ((set_model.cardinality >= other.set_model.cardinality))
			pure_feature: is_equal (old Current)
			immutable_argument: other.is_equal (old other)
			immutable_model: set_model.is_equal (old set_model)
			inverse_superset_relation: set_model.is_superset (other.set_model)
		end

	is_superset (other: SPECIFICATION_SUBSET[G]): BOOLEAN is
			-- Is current set a superset of `other'?
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_superset: other.set_model.is_superset (set_model) and Result.is_equal (other.set_model.is_subset (set_model)) and Result.is_equal ((other.set_model.cardinality >= set_model.cardinality))
			pure_feature: is_equal (old Current)
			immutable_argument: other.is_equal (old other)
			immutable_model: set_model.is_equal (old set_model)
			inverse_subset_relation: set_model.is_subset (other.set_model)
		end

	disjoint (other: SPECIFICATION_SUBSET[G]): BOOLEAN is
			-- Do current set and `other' have no items in common?
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_disjoint: Result.is_equal (set_model.is_disjoint (other.set_model)) and Result.is_equal (set_model.intersected (other.set_model).is_empty) and (set_model.is_empty or other.set_model.is_empty) implies Result
			pure_feature: is_equal (old Current)
			immutable_argument: other.is_equal (old other)
			immutable_model: set_model.is_equal (old set_model)
		end

feature -- Element change

	merge (other: SPECIFICATION_SUBSET[G]) is
			-- Add all items of `other'.
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_merge: set_model.is_equal ((old set_model).united (other.set_model)) and (old set_model).for_all (agent set_model.is_member (?)) and other.set_model.for_all (agent set_model.is_member (?))
			immutable_argument: other.is_equal (old other)
		end

feature -- Duplication

	duplicate (number: INTEGER): like Current is
			-- New structure containing min (`number', `count')
			-- items from current structure
		require
			non_negative: number >= 0
		deferred
		ensure
			definition_of_duplicate: Result.set_model.for_all (agent set_model.is_member (?)) and number <= count implies Result.set_model.cardinality.is_equal (number) and number >= count implies Result.set_model.cardinality.is_equal (count)
			pure_feature: is_equal (old Current)
			immutable_number : number.is_equal (old number)
			immutable_model: set_model.is_equal (old set_model)
		end

feature -- Basic operations

	intersect (other: SPECIFICATION_SUBSET[G]) is
			-- Remove all items not in `other'.
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_intersect: set_model.is_equal ((old set_model).intersected (other.set_model)) and set_model.for_all (agent (old set_model).is_member (?)) and set_model.for_all (agent (other.set_model).is_member (?))
			immutable_argument: other.is_equal (old other)
			existing_elements: (old set_model).filtered (agent (other.set_model).is_member (?)).is_equal (set_model)
			other_elements: other.set_model.filtered (agent (old set_model).is_member (?)).is_equal (set_model)
			existing_subset: (old set_model).is_subset (set_model)
			other_subset: other.set_model.is_subset (set_model)
		end

	subtract (other: SPECIFICATION_SUBSET[G]) is
			-- Remove all items also in `other'.
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_subtract: set_model.is_equal ((old set_model).subtracted (other.set_model)) and not other.set_model.there_exists (agent set_model.is_member (?)) and set_model.intersected (other.set_model).is_empty and set_model.for_all (agent ((old set_model).filtered (agent (other.set_model).is_member (?))).is_member (?))
			immutable_argument: other.is_equal (old other)
		end

	symdif (other: SPECIFICATION_SUBSET[G]) is
			-- Remove all items also in `other', and add all
			-- items of `other' not already present.
		require
			other_not_void: other /= Void
			same_comparison_criterion: object_comparison.is_equal (other.object_comparison)
		deferred
		ensure
			definition_of_symdif: set_model.is_equal ((old set_model).difference (other.set_model)) and set_model.is_equal ((old set_model).united (other.set_model).subtracted ((old set_model).intersected (other.set_model)))
			immutable_argument: other.is_equal (old other)
		end

invariant
	reflexive_subset_relation: is_subset (Current)
	reflexive_superset_relation: is_superset (Current)

	reflexive_disjointness_relation: disjoint (Current).is_equal (False)

end -- class SPECIFICATION_SUBSET