deferred class SPECIFICATION_TABLE [G, H]

inherit
	SPECIFICATION_BAG [G]
		rename
			put as bag_put,
			bag as bag_model
		end

	MML_SPECIFICATION

feature {MML_SPECIFICATION, ANY} -- Model

	table_model : MML_RELATION[H, G] is
			-- The table model
		deferred
		ensure
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

feature -- Access

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

	item, infix "@" (key: H): G is
			-- Entry of key `key'.
		require
			key_not_void: key /= Void
			valid_key: valid_key (key)
		deferred
		ensure
			definition_of_item: table_model.image_element (key).is_member (Result) and table_model.is_functional (key)
			pure_feature: is_equal (old Current)
			immutable_argument: key.is_equal (old key)
			immutable_models: table_model.is_equal (old table_model) and bag_model.is_equal (old bag_model)
			result_not_void: Result /= Void
		end
	
feature -- Measurement

	occurrences (other: G): INTEGER is
			-- Number of times `other' appears in structure
		deferred
		ensure then
			definition_of_occurrences: Result.is_equal (table_model.range_element_restricted (other).cardinality)
			immutable_model: table_model.is_equal (old table_model)
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		deferred
		ensure then
			definition_of_is_equal: Result.is_equal (table_model.is_equal (other.table_model))
			immutable_model: table_model.is_equal (old table_model)
		end

feature -- Status report

	changeable_comparison_criterion: BOOLEAN is
			-- May `object_comparison' be changed?
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

	is_empty: BOOLEAN is
			-- Is there no element?
		deferred
		ensure then
			definition_of_is_empty: Result.is_equal (table_model.is_empty)
			immutable_model: table_model.is_equal (old table_model)
		end

	object_comparison: BOOLEAN is
			-- Must search operations use `equal' rather than `='
			-- for comparing references? (Default: no, use `='.)
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

	valid_key (key: H): BOOLEAN is
			-- Is `key' a valid key?
		require
			key_not_void: key /= Void
		deferred
		ensure
			definition_of_valid_key: table_model.domain.is_member (key) and table_model.is_functional (key)
			pure_feature: is_equal (old Current)
			immutable_argument: key.is_equal (old key)
			immutable_models: table_model.is_equal (old table_model) and bag_model.is_equal (old bag_model)
		end

feature -- Element change

	put (value: G; key: H) is
			-- Associate value `value' with key `key'.
		require
			value_not_void: value /= void
			key_not_void: key /= void
			need_valid_key: valid_key (key)
		deferred
		ensure
			definition_of_put: table_model.is_equal (old table_model.domain_element_anti_restricted (key).extended (create {MML_DEFAULT_PAIR [H, G]}.make_from (key, value)))
			definition_of_put: bag_model.is_equal (old bag_model.extended (value))
			immutable_value: value.is_equal (old value)
			immutable_key: key.is_equal (old key)
			item_is_value: item (key).is_equal (value)
		end

feature -- Removal

	prune_all (other: G) is
			-- Remove all occurrences of `other'.
		deferred
		ensure then
			definition_prune_all: table_model.cardinality.is_equal (old table_model.range_element_anti_restricted (other).cardinality)
		end

	wipe_out is
			-- Remove all items.
		deferred
		ensure then
			definition_of_wipe_out: table_model.is_empty
		end

feature -- Status Report

	is_inserted (other: G): BOOLEAN is
			-- Has `other' been inserted by the most recent insertion?
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

	prunable: BOOLEAN is
			-- May items be removed?
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

feature -- Status Setting

	compare_objects is
			-- Ensure that future search operations will use `equal'
			-- rather than `=' for comparing references.
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

	compare_references is
			-- Ensure that future search operations will use `='
			-- rather than `equal' for comparing references.
		deferred
		ensure then
			immutable_model: table_model.is_equal (old table_model)
		end

invariant
	definition_of_table_model: table_model /= Void and table_model.is_function_valued
	definition_of_valid_key: table_model.domain.for_all (agent valid_key (?))

	glueing_invariant: table_model.range.is_superset (bag_model.domain)

end -- class SPECIFICATION_TABLE