deferred class SPECIFICATION_TO_SPECIAL [G]

inherit
	SPECIFICATION_ANY

	MML_SPECIFICATION

feature {MML_SPECIFICATION, ANY} -- Model

	sequence_model : MML_SEQUENCE[G] is
			-- The sequence model
		deferred
		ensure
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

feature -- Access

	area: SPECIAL [G] is
			-- Special data zone
		deferred
		ensure
			pure_feature: is_equal (old Current)
			immutable_model: sequence_model.is_equal (old sequence_model)
			result_not_void: Result /= Void
		end

	item, infix "@" (i: INTEGER): G is
			-- Entry at index `i', if in index interval
		require
			defined_index: sequence_model.is_defined (i)
		deferred
		ensure
			definition_of_item: Result.is_equal (sequence_model.element (i))
			pure_feature: is_equal (old Current)
			immutable_model: sequence_model.is_equal (old sequence_model)
			immutable_argument: i.is_equal (old i)
			item_in_area: Result.is_equal (area.item (i))
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		deferred
		ensure then
			definition_of_is_equal: Result.is_equal (sequence_model.is_equal (other.sequence_model))
			immutable_model: sequence_model.is_equal (old sequence_model)
		end

feature -- Status report

	valid_index (i: INTEGER): BOOLEAN is
			-- Is `i' within the bounds of Current?
		deferred
		ensure
			definition_of_valid_index: Result.is_equal (0 <= i and i < area.count)
			pure_feature: is_equal (old Current)
			immutable_model: sequence_model.is_equal (old sequence_model)
			immutable_argument: i.is_equal (old i)
		end

feature -- Element change

	put (other: G; i: INTEGER) is
			-- Replace `i'-th entry, if in index interval, by `other'.
		require
			other_not_void: other /= Void
			defined_index: sequence_model.is_defined (i)
		deferred
		ensure
			definition_of_put: sequence_model.is_equal (old sequence_model.replaced (other, i))
			immutable_element: other.is_equal (old other)
			immutable_argument: i.is_equal (old i)
		end

feature {NONE} -- Element change

	set_area (other: like area) is
			-- Make `other' the new `area'
		require
			other_not_void: other /= Void
		deferred
		ensure
			definition_of_set_area: area.is_equal (other)
			not_equal_implies_model_changed: not area.is_equal (other) implies not sequence_model.is_equal (old sequence_model)
		end

invariant
	sequence_model_not_void: sequence_model /= Void

end -- class SPECIFICATION_TO_SPECIAL