deferred class
	SPECIFICATION_TRAVERSABLE [G]

inherit
	SPECIFICATION_CONTAINER [G]

	MML_FUNCTIONALS[G]
		export
			{NONE}
				all
		end

	MML_SPECIFICATION

feature {MML_SPECIFICATION, ANY} -- Model

	sequence_model : MML_SEQUENCE[G] is
			-- The sequence model
		deferred
		ensure
			definition_of_sequence_model: Result.range.for_all (agent has (?))
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

	cursor_model : INTEGER is
			-- The cursor model
		deferred
		ensure
			definition_of_cursor_model: Result >= sequence_model.lower_bound - 1 and Result <= sequence_model.upper_bound + 1
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		deferred
		ensure then
			definition_of_is_equal: Result implies sequence_model.is_equal (other.sequence_model)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Access

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			definition_of_has: Result.is_equal (sequence_model.is_member (other))
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	item: G is
			-- Item at current position
		require
			not_off: not off
			non_empty_list : not sequence_model.is_empty
			defined_cursor_model : sequence_model.is_defined (cursor_model)
		deferred
		ensure then
			definition_of_item: Result.is_equal (sequence_model.element (cursor_model))
			pure_feature: is_equal (old Current)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Status Report

	changeable_comparison_criterion: BOOLEAN is
			-- May `object_comparison' be changed?
		deferred
		ensure then
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	is_empty: BOOLEAN is
			-- Is there no element?
		deferred
		ensure then
			definition_of_is_empty: Result.is_equal (sequence_model.is_empty)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	object_comparison: BOOLEAN is
			-- Must search operations use `equal' rather than `='
			-- for comparing references? (Default: no, use `='.)
		deferred
		ensure then
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	off: BOOLEAN is
			-- Is there no current item?
		deferred
		ensure
			definition_of_off: Result.is_equal (not sequence_model.is_defined (cursor_model))
			pure_feature: is_equal (old Current)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Cursor Movement

	start is
			-- Move to first position if any.
		deferred
		ensure
			definition_of_start: not sequence_model.is_empty implies cursor_model.is_equal (sequence_model.lower_bound)
			definition_of_start: sequence_model.is_empty implies cursor_model.is_equal (old cursor_model)
			immutable_model: sequence_model.is_equal (old sequence_model)
			empty_implies_not_defined: sequence_model.is_empty implies not sequence_model.is_defined (cursor_model)
			not_empty_implies_not_off: not is_empty implies not off
		end

feature -- Iteration

	do_all (action: PROCEDURE [ANY, TUPLE [G]]) is
			-- Apply `action' to every item.
			-- Semantics not guaranteed if `action' changes the structure;
			-- in such a case, apply iterator to clone of structure instead. 
		require
			action_not_void: action /= Void
		deferred
		ensure
			immutable_argument: action.is_equal (old action)
		end

	do_if (action: PROCEDURE [ANY, TUPLE [G]]; test: FUNCTION [ANY, TUPLE [G], BOOLEAN]) is
			-- Apply `action' to every item that satisfies `test'.
			-- Semantics not guaranteed if `action' or `test' changes the structure;
			-- in such a case, apply iterator to clone of structure instead. 
		require
			action_not_void: action /= Void
			test_not_void: test /= Void
		deferred
		ensure
			immutable_action: action.is_equal (old action)
			immutable_test: test.is_equal (old test)
		end

	there_exists (test: FUNCTION [ANY, TUPLE [G], BOOLEAN]): BOOLEAN is
			-- Is `test' true for at least one item?
		require
			test_not_void: test /= Void
		deferred
		ensure
			definition_of_there_exists: not for_all (agent negated (?, test))
			pure_feature: is_equal (old Current)
			immutable_argument: test.is_equal (old test)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			empty_implies_not_result: not is_empty implies Result
		end

	for_all (test: FUNCTION [ANY, TUPLE [G], BOOLEAN]): BOOLEAN is
			-- Is `test' true for all items?
		require
			test_not_void: test /= Void
		deferred
		ensure
			definition_of_for_all: not there_exists (agent negated (?, test))
			pure_feature: is_equal (old Current)
			immutable_argument: test.is_equal (old test)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
			empty_implies_result: is_empty implies Result
		end
	
feature -- Status Setting

	compare_objects is
			-- Ensure that future search operations will use `equal'
			-- rather than `=' for comparing references.
		deferred
		ensure then
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	compare_references is
			-- Ensure that future search operations will use `='
			-- rather than `equal' for comparing references.
		deferred
		ensure then
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

invariant
	definition_of_sequence_model: sequence_model /= Void and sequence_model.range.for_all (agent has (?))

	definition_of_linear_representation: linear_representation.for_all (agent sequence_model.is_member (?))
	empty_linear_equals_empty: linear_representation.is_empty.is_equal (sequence_model.is_empty)

	glueing_invariant: cursor_model >= sequence_model.lower_bound - 1 and cursor_model <= sequence_model.upper_bound + 1

end -- class SPECIFICATION_TRAVERSABLE