deferred class SPECIFICATION_TRAVERSABLE_SUBSET [G]

inherit
	SPECIFICATION_SUBSET [G]

	MML_SPECIFICATION

feature {MML_SPECIFICATION, ANY} -- Model

	sequence_model : MML_SEQUENCE[G] is
			-- The sequence model
		deferred
		ensure
			definition_of_sequence_model: Result.range.for_all (agent has (?))
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

	cursor_model : INTEGER is
			-- The cursor model
		deferred
		ensure
			definition_of_cursor_model: Result >= sequence_model.lower_bound - 1 and Result <= sequence_model.upper_bound + 1
			pure_feature: is_equal (old Current)
			result_not_void: Result /= Void
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		deferred
		ensure then
			result_implies_same_models: Result implies sequence_model.is_equal (other.sequence_model)
		end

feature -- Access

	item: G is
			-- Item at current position
		require
			not_off: not off
			non_empty_list : not sequence_model.is_empty
			defined_cursor_model : sequence_model.is_defined (cursor_model)
		deferred
		ensure
			definition_of_item: Result.is_equal (sequence_model.element (cursor_model))
			pure_feature: is_equal (old Current)
			immutable_models : set_model.is_equal (old set_model) and sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			definition_of_has: Result.is_equal (sequence_model.is_member (other))
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Measurement

	count: INTEGER is
			-- Number of items
		deferred
		ensure then
			definition_of_count: Result.is_equal (sequence_model.count)
			immutable_models: sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model)
		end

feature -- Status report

	after: BOOLEAN is
			-- Is there no valid position to the right of current one?
		deferred
		ensure
			definition_of_after : Result.is_equal (not sequence_model.is_defined (cursor_model) and cursor_model > sequence_model.upper_bound)
			pure_feature: is_equal (old Current)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and set_model.is_equal (old set_model)
			result_implies_off: Result implies off
		end

	off: BOOLEAN is
			-- Is there no current item?
		deferred
		ensure
			definition_of_off: Result.is_equal (not sequence_model.is_defined (cursor_model))
			pure_feature: is_equal (old Current)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and set_model.is_equal (old set_model)
			empty_or_after_implies_result: is_empty or after implies Result
		end

	is_empty: BOOLEAN is
			-- Is container empty?
		deferred
		ensure then
			definition_of_empty: Result.is_equal (count.is_equal (0))
			empty_equal_empty_models: Result.is_equal (sequence_model.is_empty) and Result.is_equal (set_model.is_empty)
			immutable_models : sequence_model.is_equal (old sequence_model) and cursor_model.is_equal (old cursor_model) and set_model.is_equal (old set_model)
			result_implies_off: Result implies off
		end

feature -- Cursor movement

	start is
			-- Move to first position if any.
		deferred
		ensure
			definition_of_start : not sequence_model.is_empty implies sequence_model.is_defined (cursor_model) and sequence_model.element (cursor_model).is_equal (sequence_model.first)
			immutable_models : sequence_model.is_equal (old sequence_model) and set_model.is_equal (old set_model)
			empty_implies_unchanged : sequence_model.is_empty implies (cursor_model.is_equal (old cursor_model))
			not_defined_implies_after : not sequence_model.is_empty implies (not sequence_model.is_defined (cursor_model) and cursor_model > sequence_model.upper_bound)
			not_empty_implies_not_off: not is_empty implies not off
		end

	forth is
			-- Move to next position.
		require
			not_after: not after
			index_less_upper_bound : cursor_model <= sequence_model.upper_bound
		deferred
		ensure
			definition_of_forth : sequence_model.is_defined (cursor_model) implies cursor_model.is_equal (old cursor_model + 1)
			immutable_models : sequence_model.is_equal (old sequence_model) and set_model.is_equal (old set_model)
			not_defined_implies_after : not sequence_model.is_defined (cursor_model) implies cursor_model > sequence_model.upper_bound
		end

feature -- Removal

	remove is
			-- Remove current item.
		require
			defined_cursor : sequence_model.is_defined (cursor_model)
		deferred
		ensure
			definition_of_remove : sequence_model.is_equal ((old sequence_model).pruned_at (cursor_model)) and set_model.is_equal ((old set_model).pruned ((old sequence_model).element (cursor_model)))
			count_decreased : sequence_model.count.is_equal ((old sequence_model).count - 1)
		end

invariant
	definition_of_sequence_model: sequence_model /= Void and sequence_model.range.for_all (agent has (?))
	definition_of_cursor_model: cursor_model >= sequence_model.lower_bound - 1 and cursor_model <= sequence_model.upper_bound + 1

	glueing_invariant: sequence_model.range.is_equal (set_model)

	empty_implies_zero_count: is_empty.is_equal (count.is_equal (0))
	non_negative_count: count >= 0

end -- class SPECIFICATION_TRAVERSABLE_SUBSET