deferred class SPECIFICATION_BAG [G]

inherit
	SPECIFICATION_COLLECTION [G]

	MML_SPECIFICATION

feature {MML_SPECIFICATION, ANY} -- Model

	bag : MML_BAG[G] is
			-- The bag model
		deferred
		ensure
			is_equal (old Current)
			Result /= Void
		end

feature -- Access

	has (other: G): BOOLEAN is
			-- Does structure include `other'?
		deferred
		ensure then
			Result.is_equal (bag.is_member (other))
			bag.is_equal (old bag)
		end

feature -- Measurement

	occurrences (other: G): INTEGER is
			-- Number of times `other' appears in structure
		require
			other /= Void
		deferred
		ensure
			Result.is_equal (bag.multiplicity (other))
			is_equal (old Current)
			bag.is_equal (old bag)
			other.is_equal (old other)
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered equal to current object?
		deferred
		ensure then
			Result.is_equal (bag.is_equal (other.bag))
			bag.is_equal (old bag)
		end
	
feature -- Status report

	changeable_comparison_criterion: BOOLEAN is
			-- May `object_comparison' be changed?
		deferred
		ensure then
			bag.is_equal (old bag)
		end

	is_empty: BOOLEAN is
			-- Is there no element?
		deferred
		ensure then
			Result.is_equal (bag.is_empty)
			bag.is_equal (old bag)
		end

	object_comparison: BOOLEAN is
			-- Must search operations use `equal'?
		deferred
		ensure then
			bag.is_equal (old bag)
		end

feature -- Element change

	extend (other: G) is
			-- Add a new occurrence of `other'.
		deferred
		ensure then
			bag.is_equal (old bag.extended (other))
			bag.multiplicity (other).is_equal (old bag.multiplicity (other))
		end

feature -- Removal

	prune (other: G) is
			-- Remove one occurrence of `other' if any.
		deferred
		ensure then
			bag.is_equal (old bag.pruned (other))
		end

	prune_all (other: G) is
			-- Remove all occurrences of `other'.
		deferred
		ensure then
			bag.is_equal (old bag.domain_element_anti_restricted (other))
		end

	wipe_out is
			-- Remove all items.
		deferred
		ensure then
			bag.is_empty
		end

feature -- Element Change

	put (other: G) is
			-- Ensure that structure includes `other'.
		deferred
		ensure then
			bag.is_equal (old bag.extended (other))
			bag.multiplicity (other).is_equal (old bag.multiplicity (other))
		end

feature -- Status Report

	extendible: BOOLEAN is
			-- May new items be added?)
		deferred
		ensure then
			bag.is_equal (old bag)
		end

	is_inserted (other: G): BOOLEAN is
			-- Has `other' been inserted by the most recent insertion?
		deferred
		ensure then
			bag.is_equal (old bag)
		end

	prunable: BOOLEAN is
			-- May items be removed?
		deferred
		ensure then
			bag.is_equal (old bag)
		end

feature -- Status Setting

	compare_objects is
			-- Ensure that future search operations will use `equal'
		deferred
		ensure then
			bag.is_equal (old bag)
		end

	compare_references is
			-- Ensure that future search operations will use `='
		deferred
		ensure then
			bag.is_equal (old bag)
		end

invariant
	linear_representation.for_all (agent bag.is_member (?))
	linear_representation.is_empty.is_equal (bag.is_empty)

end -- class SPECIFICATION_BAG