deferred class SPECIFICATION_QUEUE [G]

inherit
	SPECIFICATION_DISPENSER [G]
		rename
			cursor_model as cursor,
			bag_model as bag,
			sequence_model as sequence
		export
			{NONE} prune, prune_all
		end

feature -- Removal

	remove is
			-- Remove current item.
		deferred
		ensure then
			not sequence.is_empty implies cursor.is_equal (old cursor)
		end

feature -- Conversion

	linear_representation: SPECIFICATION_LINEAR [G] is
			-- Representation as a linear structure
		deferred
		ensure then
			Result.sequence_model.is_equal (sequence)
			sequence.is_equal (old sequence)
			cursor.is_equal (old cursor)
			bag.is_equal (old bag)
		end

feature -- Element change

	append (other: SPECIFICATION_SEQUENCE [G]) is
			-- Append a copy of `other'.
		deferred
		ensure then
			cursor.is_equal (old cursor)
		end

	 force (other: G) is
			-- Add `other' as newest item.
		require
			other_not_void: other /= Void
		deferred
		ensure
			sequence.is_equal (old sequence.appended (other))
			bag.is_equal (old bag.extended (other))
			other.is_equal (old other)
			cursor.is_equal (old cursor)
		end

	put (other: G) is
			-- Ensure that structure includes `other'.
		deferred
		ensure then
			sequence.is_equal (old sequence.appended (other))
			cursor.is_equal (old cursor)
		end

	extend (other: G) is
			-- Add a new occurrence of `other'.
		deferred
		ensure then
			sequence.is_equal (old sequence.appended (other))
			cursor.is_equal (old cursor)
		end

invariant
	not sequence.is_empty implies cursor.is_equal (sequence.lower_bound)
	sequence.is_empty implies cursor.is_equal (sequence.lower_bound - 1)

end -- class SPECIFICATION_QUEUE