deferred class SPECIFICATION_STACK [G]

inherit
	SPECIFICATION_DISPENSER [G]
		rename
			cursor_model as cursor,
			bag_model as bag,
			sequence_model as sequence
		export
			{NONE} prune, prune_all
		end
	
feature -- Removal

	remove is
			-- Remove current item.
		deferred
		ensure then
			cursor.is_equal (old cursor - 1)
		end

feature -- Conversion

	linear_representation: SPECIFICATION_LINEAR [G] is
			-- Representation as a linear structure
		deferred
		ensure then
			Result.sequence_model.is_equal (sequence)
			sequence.is_equal (old sequence)
			cursor.is_equal (old cursor)
			bag.is_equal (old bag)
		end

feature -- Element change

	append (other: SPECIFICATION_SEQUENCE [G]) is
			-- Append a copy of `other'.
		deferred
		ensure then
			cursor.is_equal (old cursor + other.sequence_model.count)
		end

	extend (other: G) is
			-- Push `other' onto top.
		deferred
		ensure then
			sequence.is_equal (old sequence.appended (other))
			cursor.is_equal (old cursor + 1)
		end

	replace (other: G) is
			-- Replace top item by `other'.
		deferred
		ensure then
			sequence.is_equal (old sequence.front.appended (other))
		end

invariant
	not sequence.is_empty implies cursor.is_equal (sequence.upper_bound)

end -- class SPECIFICATION_STACK